// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef CMACHINE_H
#define CMACHINE_H

#include "BaseTypes.h"

#include "CMachineNode.h"
#include "CMachineConn.h"

/*! @brief Machine object
 * 
 * Represents a generic "machine" - basically the idea is to simulate an 
 * arbitrarily complex set of nodes with position and movement that can be
 * connected to one another with connections of arbitrary lengths.
 *
 */
 
class CMachine
{
    public:

        /*! Default constructor */
        CMachine();
        
        /*! Virtual deconstructor */
        virtual ~CMachine();

        /*! Access the machine's list of nodes */
        CMachineNodeList *GetNodeList() { return this->pNodeH; }

        /*! Access the machine's list of connections */
        CMachineConnList *GetConnList() { return this->pConnH; }

    protected:

        /*! Insert a node */
        void InsertNode(CMachineNodeList *pNode);

        /*! Insert a connection */
        void InsertConn(CMachineConnList *pConn);

        /*! This machine's head nodes */
        CMachineNodeList *pNodeH;

        /*! This machine's tail node */
        CMachineNodeList *pNodeT;

        /*! This machine's head connection */
        CMachineConnList *pConnH;

        /*! This machine's tail connection */
        CMachineConnList *pConnT;
};

#endif
