// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef CMACHINECONN_H
#define CMACHINECONN_H

#include "BaseTypes.h"

#include "CMachineNode.h"

/*! @brief Machine connection
 * 
 * Represents a generic "connection" - this object acts as a way to connect
 * two nodes together with a line of the given length.
 *
 */

class CMachineConn
{
    public:

        /*! Constructor, connects pNode0 to pNode1 with a connector of length fLength */
        CMachineConn(CMachineNode *pNode0, CMachineNode *pNode1, float fLength) : pNode0(pNode0), pNode1(pNode1), fLength(fLength) { }

        /*! Access node 0 */
        CMachineNode *GetNode0() { return this->pNode0; }

        /*! Access node 1 */
        CMachineNode *GetNode1() { return this->pNode1; }

        /*! Access the connection length */
        float GetLength() { return this->fLength; }

    protected:

        /*! This connection's node 0 */
        CMachineNode *pNode0;

        /*! This connection's node 1 */
        CMachineNode *pNode1;

        /*! This connection's length */
        float fLength;
};

/*! @brief Linked-list of machine connections */
class CMachineConnList : public CMachineConn
{
    public:

        /*! Constructor, creates a generic "connection" along with a link to continue this list */
        CMachineConnList(CMachineNode *pNode0, CMachineNode *pNode1, float fLength) : CMachineConn(pNode0, pNode1, fLength), pNext(0) { }

        /*! Next connection in this linked list */
        CMachineConnList *pNext;
};

#endif
