// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef CMACHINENODE_H
#define CMACHINENODE_H

#include "BaseTypes.h"

/*! @brief Machine node
 * 
 * Represents a generic "node" - this object maintains the position and
 * acceleration of a single node,
 *
 */

class CMachineNode
{
    public:
        
        /*! Constructor, initializes node with the given X,Y and acceleration values */
        CMachineNode(float fPosX = 0, float fPosY = 0, float fPosXs = 0, float fPosYs = 0) : fPosX(fPosX), fPosY(fPosY), fPosXs(fPosXs), fPosYs(fPosYs) { }

        /*!
         * @param fPosX The X component of new position.
         * @param fPosY The Y component of new position.
         */
        void SetPos(float fPosX, float fPosY) { this->fPosX = fPosX; this->fPosY = fPosY; }

        /*! Access position in X direction */
        float GetPosX() { return this->fPosX; }

        /*! Access position in Y direction */
        float GetPosY() { return this->fPosY; }

        /*!
         * @param fPosXs The X component of new acceleration.
         * @param fPosYs The Y component of new acceleration.
         */
        void SetPoss(float fPosXs, float fPosYs) { this->fPosXs = fPosXs; this->fPosYs = fPosYs; }

        /*! Access acceleration in X direction */
        float GetPosXs() { return this->fPosXs; }

        /*! Access acceleration in Y direction */
        float GetPosYs() { return this->fPosYs; }

    protected:

        // Position (0.0 -> 1.0)
        float fPosX,  fPosY;

        // Movement per unit time
        float fPosXs, fPosYs;
};

/*! @brief Linked-list of machine nodes */
class CMachineNodeList : public CMachineNode
{
    public:
        
        /*! Constructor, creates a generic "node" along with a link to continue this list */
        CMachineNodeList(float fPosX = 0, float fPosY = 0, float fPosXs = 0, float fPosYs = 0) : CMachineNode(fPosX, fPosY, fPosXs, fPosYs), pNext(0) { }

        /*! Next connection in this linked list */
        CMachineNodeList *pNext;
};

#endif
