// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef CMACHINEWORLD_H
#define CMACHINEWORLD_H

#include "BaseTypes.h"

#include "CMachine.h"
#include "CMachine_Box.h"
#include "CMachine_Grid.h"
#include "CMachine_Chain.h"
#include "CMachine_Triangle.h"

#include <list>

/*! @brief Machine environment
 * 
 * Represents an entire "world" of machines. Manages the movement of nodes, the
 * affect of gravity, etc. Time can be incremented at a variable rate.
 *
 */

class CMachineWorld
{
    public:

        /*! Default constructor */
        CMachineWorld();
        
        /*! Virtual deconstructor */
        virtual ~CMachineWorld();

        /*! Insert a single machine into the "world" */
        void InsertMachine(CMachine *pMachine) { this->MachineList.push_back(pMachine); };

        /*! Advance time by the given rate. */
        void IncrementTime(float fStep = 1.0f);

        /*!
         * @param fGravX The X component of new gravity.
         * @param fGravY The Y component of new gravity.
         */
        void SetGravity(float fGravX, float fGravY) { this->fGravX = fGravX; this->fGravY = fGravY; }

        /*! Access the world's X component of gravity */
        float GetGravX() { return this->fGravX; }

        /*! Access the world's Y component of gravity */
        float GetGravY() { return this->fGravY; }

    protected:

        /*! list of machines existing in the "world" */
        std::list<CMachine *> MachineList;

        /*! local cache of gravity in the "world" */
        float fGravX, fGravY;
};

#endif
