// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CMachine_Box.h"

#include <math.h>

CMachine_Box::CMachine_Box(float fPosX, float fPosY, float fSideLength, bool bReinforced) : CMachine()
{
    /*! create our standard 4 nodes */
    CMachineNodeList *pNode0 = new CMachineNodeList(fPosX - fSideLength/2, fPosY - fSideLength/2);
    CMachineNodeList *pNode1 = new CMachineNodeList(fPosX + fSideLength/2, fPosY - fSideLength/2);
    CMachineNodeList *pNode2 = new CMachineNodeList(fPosX - fSideLength/2, fPosY + fSideLength/2);
    CMachineNodeList *pNode3 = new CMachineNodeList(fPosX + fSideLength/2, fPosY + fSideLength/2);

    InsertNode(pNode0);
    InsertNode(pNode1);
    InsertNode(pNode2);
    InsertNode(pNode3);

    /*! connect our 4 nodes to form a box */
    CMachineConnList *pConn0 = new CMachineConnList(pNode0, pNode1, fSideLength);
    CMachineConnList *pConn1 = new CMachineConnList(pNode0, pNode2, fSideLength);
    CMachineConnList *pConn2 = new CMachineConnList(pNode1, pNode3, fSideLength);
    CMachineConnList *pConn3 = new CMachineConnList(pNode2, pNode3, fSideLength);

    InsertConn(pConn0);
    InsertConn(pConn1);
    InsertConn(pConn2);
    InsertConn(pConn3);

    /*! optionally reinforce the box by linking 2 more sets of nodes */
    if(bReinforced)
    {
        float fLength = sqrtf(fSideLength*fSideLength + fSideLength*fSideLength);

        pConnH->pNext->pNext->pNext->pNext = new CMachineConnList(pNode0, pNode3, fLength);
        pConnH->pNext->pNext->pNext->pNext->pNext = new CMachineConnList(pNode1, pNode2, fLength);
    }
}
