// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CMachine_Chain.h"

#include <math.h>

CMachine_Chain::CMachine_Chain(float fPosX, float fPosY, float fConnLengthX, float fConnLengthY, uint32 uiNodeCount) : CMachine()
{
    CMachineNodeList *pNodeL = 0;

    /*! calculate initial X,Y offsets in order to center the chain */
    float fCurOffsX = (fConnLengthX * (uiNodeCount-1)) / 2.0f;
    float fCurOffsY = (fConnLengthY * (uiNodeCount-1)) / 2.0f;

    /*! calculate the line length required to move in the given X,Y direction */
    float fLineLength = (float)hypot(fConnLengthX, fConnLengthY);

    /*! iterate through the specified number of nodes */
    for(uint32 n=0;n<uiNodeCount;n++)
    {
        CMachineNodeList *pCurNode = new CMachineNodeList(fPosX + fCurOffsX, fPosY + fCurOffsY);

        fCurOffsX -= fConnLengthX;
        fCurOffsY -= fConnLengthY;

        InsertNode(pCurNode);

        if(pNodeL != 0)
        {
            CMachineConnList *pCurConn = new CMachineConnList(pNodeL, pCurNode, fLineLength);

            InsertConn(pCurConn);
        }

        pNodeL = pCurNode;
    }
}
