// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CMachine_Grid.h"

#include <math.h>

CMachine_Grid::CMachine_Grid(float fPosX, float fPosY, float fConnLengthX, float fConnLengthY, uint32 uiNodeCountX, uint32 uiNodeCountY) : CMachine()
{
    /*! previous node */
    CMachineNodeList *pNodeL = 0;
    /*! current node */
    CMachineNodeList *pNodeC = 0;
    /*! previous tail node */
    CMachineNodeList *pNodeP = 0;

    /*! calculate initial X,Y offsets in order to center the grid */
    float fCurOffsX = (fConnLengthX * (uiNodeCountX-1)) / 2.0f;
    float fCurOffsY = (fConnLengthY * (uiNodeCountY-1)) / 2.0f;

    /*! calculate the line length required to move in the given X,Y direction */
    float fLineLength = (float)hypot(fConnLengthX, fConnLengthY);

    uint32 m, n;

    /*! generate grid of nodes */
    for(m=0;m<uiNodeCountY;m++)
    {
        for(n=0;n<uiNodeCountX;n++)
        {
            CMachineNodeList *pCurNode = new CMachineNodeList(fPosX + fCurOffsX, fPosY + fCurOffsY);

            fCurOffsX -= fConnLengthX;

            InsertNode(pCurNode);
        }

        fCurOffsY -= fConnLengthY;
        fCurOffsX += fConnLengthX * uiNodeCountX;
    }

    pNodeC = pNodeH;
    pNodeP = 0;

    /*! generate node connections */
    for(m=0;m<uiNodeCountY;m++)
    {
        CMachineNodeList *pNodeCache = pNodeC;

        pNodeL = 0;

        for(n=0;n<uiNodeCountX;n++)
        {
            if(pNodeP != 0)
            {
                CMachineConnList *pCurConn = new CMachineConnList(pNodeP, pNodeC, fConnLengthY);

                InsertConn(pCurConn);

                if(pNodeP->pNext != 0 && (n+1)%uiNodeCountX != 0)
                {
                    pCurConn = new CMachineConnList(pNodeP->pNext, pNodeC, fLineLength);

                    InsertConn(pCurConn);
                }

                if(pNodeC->pNext != 0 && (n+1)%uiNodeCountX != 0)
                {
                    pCurConn = new CMachineConnList(pNodeP, pNodeC->pNext, fLineLength);

                    InsertConn(pCurConn);
                }

                pNodeP = pNodeP->pNext;
            }

            if(pNodeL != 0)
            {
                CMachineConnList *pCurConn = new CMachineConnList(pNodeL, pNodeC, fConnLengthX);

                InsertConn(pCurConn);
            }

            pNodeL = pNodeC;
            pNodeC = pNodeC->pNext;
        }

        pNodeP = pNodeCache;
    }
}
