// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CMachine_Triangle.h"

#include <math.h>

CMachine_Triangle::CMachine_Triangle(float fPosX, float fPosY, float fConnLength, float fRotation) : CMachine()
{
    float fDeltaX[3] = 
    {
        sinf(fRotation + 0.00f),
        sinf(fRotation + 2.09f),
        sinf(fRotation + 4.18f),
    };

    float fDeltaY[3] = 
    {
        cosf(fRotation + 0.00f),
        cosf(fRotation + 2.09f),
        cosf(fRotation + 4.18f),
    };

    /*! create our 3 nodes */
    CMachineNodeList *pNode0 = new CMachineNodeList(fPosX + fDeltaX[0]*fConnLength, fPosY + fDeltaY[0]*fConnLength);
    CMachineNodeList *pNode1 = new CMachineNodeList(fPosX + fDeltaX[1]*fConnLength, fPosY + fDeltaY[1]*fConnLength);
    CMachineNodeList *pNode2 = new CMachineNodeList(fPosX + fDeltaX[2]*fConnLength, fPosY + fDeltaY[2]*fConnLength);

    InsertNode(pNode0);
    InsertNode(pNode1);
    InsertNode(pNode2);

    /*! connect our 3 nodes to form a triangle */
    float chkX0 = sinf(0.00f)*fConnLength;
    float chkY0 = cosf(0.00f)*fConnLength;
    float chkX1 = sinf(2.09f)*fConnLength;
    float chkY1 = cosf(2.09f)*fConnLength;

    float dX = chkX1 - chkX0;
    float dY = chkY1 - chkY0;

    float fLineLength = sqrtf(dX*dX + dY*dY);

    CMachineConnList *pConn0 = new CMachineConnList(pNode0, pNode1, fLineLength);
    CMachineConnList *pConn1 = new CMachineConnList(pNode1, pNode2, fLineLength);
    CMachineConnList *pConn2 = new CMachineConnList(pNode2, pNode0, fLineLength);

    InsertConn(pConn0);
    InsertConn(pConn1);
    InsertConn(pConn2);
}
