// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef CMACHINE_TRIANGLE_H
#define CMACHINE_TRIANGLE_H

#include "CMachine.h"

/*! @brief Triangle machine
 * 
 * Represents a "Triangle" machine. This type of machine consists of 3 nodes
 * and 3 connectors. The shape of this machine is initially a triangle.
 *
 * With zero rotation,the triangle is in the following orientation:
 *
 * <pre>
 *      /\
 *     /  \
 *    /    \
 *   /______\
 *
 * </pre>
 *
 */

class CMachine_Triangle : public CMachine
{
    public:

        /*! 
         * 
         * @param fPosX The center X position of this triangle.
         * @param fPosY The center Y position of this triangle.
         * @param fConnLength The length of each connector,
         * @param fRotation The rotation of the triangle (in radians).
         *
         */

        CMachine_Triangle(float fPosX, float fPosY, float fConnLength = 0.05f, float fRotation = 0.0f);
};

#endif
