// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CRender.h"

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "jsapi.h"

CRender::CRender(uint width, uint height) : width(width), height(height)
{
    nextPal = CRENDER_PALETTE_RANDOM_SMOOTH_HUE;
    
    bStereogram = false;
}

void CRender::hsv2rgb(float fH, float fS, float fV, float *fR, float *fG, float *fB)
{
    fH = fmodf(fH, 360.0f) / 60.0f;

    float fDiff = fH - (int)fH;
    float fM, fN, fK;

    fM = fV * (1.0f - fS);
    fN = fV * (1.0f - fS * fDiff);
    fK = fV * (1.0f - fS * (1.0f - fDiff));

    switch((int)fH)
    {
        case 0: *fR = fV; *fG = fK; *fB = fM; break;
        case 1: *fR = fN; *fG = fV; *fB = fM; break;
        case 2: *fR = fM; *fG = fV; *fB = fK; break;
        case 3: *fR = fM; *fG = fN; *fB = fV; break;
        case 4: *fR = fK; *fG = fM; *fB = fV; break;
        case 5: *fR = fV; *fG = fM; *fB = fN; break;
    }

    return;
}

void CRender::FadePaletteFL(float r0, float g0, float b0, float r1, float g1, float b1, FadePaletteCallback pCallback, void *pContext, int first, int last)
{
    float stepR = (r1 - r0) / (float)(last - first);
    float stepG = (g1 - g0) / (float)(last - first);
    float stepB = (b1 - b0) / (float)(last - first);

    for(int v=first;v<=last;v++)
    {
        pCallback(pContext, v, r0, g0, b0);

        r0 += stepR;
        g0 += stepG;
        b0 += stepB;
    }

    return;
}

void CRender::FadePaletteDW(uint32 r0, uint32 g0, uint32 b0, uint32 r1, uint32 g1, uint32 b1, FadePaletteCallback pCallback, void *pContext, int first, int last)
{
    FadePaletteFL
    (
        (float)r0 / 255.0f,
        (float)g0 / 255.0f,
        (float)b0 / 255.0f,
        (float)r1 / 255.0f,
        (float)g1 / 255.0f,
        (float)b1 / 255.0f,
        pCallback, pContext, first, last
    );

    return;
}

static JSBool global_enumerate(JSContext *cx, JSObject *obj)
{
    return JS_TRUE;
}

static JSBool global_resolve(JSContext *cx, JSObject *obj, jsval id, uintN flags, JSObject **objp)
{
    return JS_TRUE;
}

void CRender::GeneratePalette(FadePaletteCallback pCallback, void *pContext)
{
    srand(time(NULL));

    /*! load javascript */
    if(false)
    {
        JSRuntime *rt = JS_NewRuntime(0x400000);

        if(rt != 0)
        {
            JSContext *cx = JS_NewContext(rt, 8192);

            if(cx != 0)
            {
                JSClass gc = 
                {
                    "global", JSCLASS_NEW_RESOLVE,
                    JS_PropertyStub,  JS_PropertyStub,
                    JS_PropertyStub,  JS_PropertyStub,
                    &global_enumerate, (JSResolveOp) global_resolve,
                    JS_ConvertStub,   JS_FinalizeStub
                };

                JSObject *globalObj = JS_NewObject(cx, &gc, 0, 0);

                JS_InitStandardClasses(cx, globalObj);

                /*! load source */
                {
                    jsval rval;

                    char *source = "1 + 3";

                    JSBool ok = JS_EvaluateScript(cx, globalObj, source, strlen(source), "C:\\", 0, &rval);

                    if(ok)
                    {
                        jsdouble d = 0;

                        ok = JS_ValueToNumber(cx, rval, &d);
                    }
                }

                JS_DestroyContext(cx);
            }

            JS_DestroyRuntime(rt);
        }
    }

    int pick = nextPal;

    /*! select a random palette */
    if(pick == CRENDER_PALETTE_RANDOM) 
    { 
        pick = rand()%CRENDER_PALETTE_COUNT;
    }

    switch(pick)
    {
        /*! random smooth hue palette */
        case CRENDER_PALETTE_RANDOM_SMOOTH_HUE:
        {
            float fR, fG, fB;
            float fH, fS, fV;

            int i;

            fH = (float)(rand()%256);

            for(i=0;i<256;i++)
            {
                float fX = (float)i / 256.0f;
                float fY = powf(fX, 0.3f) + sinf(fX*3.1415f*2)/4.0f;

                /*! clamp fY to keep in range */
                if(fY > 1.0f) { fY = 1.0f; }
                if(fY < 0.0f) { fY = 0.0f; }

                fS = 1.0f - fY;
                fV = 0.0f + fY;

                hsv2rgb(fH, fS, fV, &fR, &fG, &fB);

                pCallback(pContext, i, fR, fG, fB);
            }
        }
        break;

        /*! Broken TV interference palette ;] */
        case CRENDER_PALETTE_BROKEN_TV:
        {
            float fR, fG, fB;
            float fH, fS, fV;

            int i;

            for(i=0;i<256;i++)
            {
                fH = (float)(rand()%256);
                fS = 1.0f - (float)i / 255.0f;
                fV = 0.0f + (float)i / 255.0f;

                hsv2rgb(fH, fS, fV, &fR, &fG, &fB);

                pCallback(pContext, i, fR, fG, fB);
            }
        }
        break;

        /*! random polarized hue palette */
        case CRENDER_PALETTE_RANDOM_POLARIZED_HUE:
        {
            float fR, fG, fB;
            float fH, fS, fV;

            int i;

            fH = (float)(rand()%256);

            for(i=0;i<256;i++)
            {
                float fX0 = (float)i / 256.0f;
                float fX1 = (float)((i*2)%256) / 255.0f;

                float fY = powf(fX1, 0.3f) + sinf(fX1*3.1415f*2)/4.0f;

                fS = 1.0f - fX0;
                fV = 0.0f + fX1;

                hsv2rgb(fH, fS, fV, &fR, &fG, &fB);

                pCallback(pContext, i, fR, fG, fB);
            }
        }
        break;

        /*! random polarized hue palette #2 */
        case CRENDER_PALETTE_RANDOM_POLARIZED_HUE2:
        {
            float fR, fG, fB;
            float fH, fS, fV;

            int i;

            fH = (float)(rand()%256);

            for(i=0;i<256;i++)
            {
                float fX = (float)((i*2)%256) / 255.0f;

                fS = 1.0f - fX;
                fV = 0.0f + fX;

                hsv2rgb(fH, fS, fV, &fR, &fG, &fB);

                pCallback(pContext, i, fR, fG, fB);
            }
        }
        break;
    }

    return;
}
