// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CRenderDD4_FS.h"

CRenderDD4_FS::CRenderDD4_FS(uint width, uint height, HWND hWnd) : CRender(width, height), hWnd(hWnd)
{
    pDirectDraw = 0;
    pDirectDraw4 = 0;
    pDD4SPrimary = 0;
    pDD4SBackBuffer = 0;
    rotPaletteOffset = 0;

    pColorTable = new PALETTEENTRY[256];
    pColorTableBack = new PALETTEENTRY[256];
}

CRenderDD4_FS::~CRenderDD4_FS()
{
    ShowCursor(TRUE);

    if(pDD4SPrimary != NULL)
    {
        pDD4SPrimary->Release();
        pDD4SPrimary = 0;
    }

    if(pDirectDraw4 != NULL)
    {
        pDirectDraw4->Release();
        pDirectDraw4 = 0;
    }

    if(pDirectDraw != NULL)
    {
        pDirectDraw->Release();
        pDirectDraw = 0;
    }

    if(pColorTable != 0)
    {
        delete[] pColorTable;
        pColorTable = 0;
    }

    if(pColorTableBack != 0)
    {
        delete[] pColorTableBack;
        pColorTableBack = 0;
    }
}

bool CRenderDD4_FS::Create()
{
    HRESULT hRet = DirectDrawCreate(NULL, &pDirectDraw, NULL);

    if(FAILED(hRet))
    {
        return false;
    }

    hRet = pDirectDraw->QueryInterface(IID_IDirectDraw4, (PVOID*)&pDirectDraw4);

    if(FAILED(hRet))
    {
        return false;
    }

    hRet = pDirectDraw4->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);

    if(FAILED(hRet))
    {
        return false;
    }

    hRet = pDirectDraw4->SetDisplayMode(width, height, 8, 0, 0);

    if(FAILED(hRet))
    {
        return false;
    }

    ShowCursor(FALSE);

    DDSURFACEDESC2 surfaceDesc;

    ZeroMemory(&surfaceDesc, sizeof(DDSURFACEDESC2));

    surfaceDesc.dwSize = sizeof(DDSURFACEDESC2);
    surfaceDesc.dwFlags = DDSD_BACKBUFFERCOUNT | DDSD_CAPS;
    surfaceDesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
    surfaceDesc.dwBackBufferCount = 1;

    hRet = pDirectDraw4->CreateSurface(&surfaceDesc, &pDD4SPrimary, NULL);

    if(FAILED(hRet))
    {
        return false;
    }

    DDSCAPS2 ddsCaps;

    ZeroMemory(&ddsCaps, sizeof(DDSCAPS2));

    ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;

    hRet = pDD4SPrimary->GetAttachedSurface(&ddsCaps, &pDD4SBackBuffer);

    if(FAILED(hRet))
    {
        return false;
    }

    //
    // initialize palette front/back
    //

    GeneratePalette(fpcRGB32, pColorTable);
    GeneratePalette(fpcRGB32, pColorTableBack);

    hRet = pDirectDraw4->CreatePalette(DDPCAPS_8BIT | DDPCAPS_ALLOW256, pColorTable, &pDDPalette, NULL);

    if(FAILED(hRet))
    {
        return false;
    }

    hRet = pDD4SPrimary->SetPalette(pDDPalette);

    if(FAILED(hRet))
    {
        return false;
    }

    return true;
}

bool CRenderDD4_FS::RenderPixels(uint08 *pPixels)
{
    DDSURFACEDESC2 SurfaceDesc;

    ZeroMemory(&SurfaceDesc, sizeof(DDSURFACEDESC2));
    
    SurfaceDesc.dwSize = sizeof(DDSURFACEDESC2);

    HRESULT hRet = pDD4SBackBuffer->Lock(NULL, &SurfaceDesc, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL);

    if(FAILED(hRet))
    {
        return false;
    }

    // @todo: Sys->Vid hardware blit (could cache several sys surfaces, or create each time)

    uint08 *pix8 = (uint08*)SurfaceDesc.lpSurface;
    uint32  wadd = SurfaceDesc.lPitch;

    int y, w = width, h = height;

    for(y=0;y<h;y++)
    {
        memcpy(pix8, pPixels, w);

        pix8 += wadd;
        pPixels += w;
    }

    pDD4SBackBuffer->Unlock(NULL);

	for(;;)
	{
        hRet = pDD4SPrimary->Flip(NULL, 0);

        if(hRet == DD_OK)
        {
            break;
        }

        if(hRet == DDERR_SURFACELOST)
		{
			hRet = pDD4SPrimary->Restore();

			if(hRet != DD_OK)
            {
				break;
            }
		}

        if(hRet != DDERR_WASSTILLDRAWING)
        {
		    break;
        }
	}

    return true;
}

bool CRenderDD4_FS::RotatePalette()
{
    if(rotPaletteOffset < 256*5)
    {
        //
        // NOTE: Solar Rotation
        //

        if(rotPaletteOffset%5 == 0)
        {
            for(int r=0;r<256;r++)
            {
                uint32 rs = pColorTableBack[r].peRed;
                uint32 gs = pColorTableBack[r].peGreen;
                uint32 bs = pColorTableBack[r].peBlue;

                uint32 rd = pColorTable[r].peRed;
                uint32 gd = pColorTable[r].peGreen;
                uint32 bd = pColorTable[r].peBlue;

                if(rd > rs) { rd -= 0x00010000; }
                else if(rd < rs) { rd += 0x00010000; }

                if(gd > gs) { gd -= 0x00000100; }
                else if(gd < gs) { gd += 0x00000100; }

                if(bd > bs) { bd -= 0x00000001; }
                else if(bd < bs) { bd += 0x00000001; }

                pColorTable[r].peRed = (BYTE)rd;
                pColorTable[r].peGreen = (BYTE)gd;
                pColorTable[r].peBlue = (BYTE)bd;
                pColorTable[r].peFlags = 0;
            }
        }

        pDDPalette->SetEntries(0, 0, 256, pColorTable);
    }
    else if(rotPaletteOffset == 256*6)
    {
        GeneratePalette(fpcRGB32, pColorTableBack);

        rotPaletteOffset = -1;
    }

    rotPaletteOffset++;

    return true;
}

void CRenderDD4_FS::fpcRGB32(void *pContext, uint32 i, float r, float g, float b)
{
    PALETTEENTRY *pColorTable = (PALETTEENTRY*)pContext;

    uint08 uiR = (uint08)(r*255.0f);
    uint08 uiG = (uint08)(g*255.0f);
    uint08 uiB = (uint08)(b*255.0f);

    pColorTable[i].peRed = uiR;
    pColorTable[i].peGreen = uiB;
    pColorTable[i].peBlue = uiG;

    return;
}
