// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "CRenderDX9.h"

CRenderDX9::CRenderDX9(uint width, uint height, HWND hWnd) : CRender(width, height), hWnd(hWnd)
{
    pDirect3D9 = NULL;
    pDirect3DDevice9 = NULL;
    rotPaletteOffset = 0;

    pColorTable = new uint32[256];
    pColorTableBack = new uint32[256];
}

CRenderDX9::~CRenderDX9()
{
    if(pDirect3DDevice9 != NULL)
    {
        pDirect3DDevice9->Release();
        pDirect3DDevice9 = NULL;
    }

    if(pDirect3D9 != NULL)
    {
        pDirect3D9->Release();
        pDirect3D9 = NULL;
    }

    if(pColorTable != 0)
    {
        delete[] pColorTable;
    }

    if(pColorTableBack != 0)
    {
        delete[] pColorTableBack;
    }
}

bool CRenderDX9::Create()
{
    pDirect3D9 = Direct3DCreate9(D3D_SDK_VERSION);

    if(pDirect3D9 == NULL)
    {
        return false;
    }

    HRESULT hRet;

    /*! attempt to triple/double buffer, if possible */
    for(int b=3;b>0;b--)
    {
        D3DPRESENT_PARAMETERS PresentationParameters;

        PresentationParameters.BackBufferWidth = width;
        PresentationParameters.BackBufferHeight = height;
        PresentationParameters.BackBufferFormat = D3DFMT_X8R8G8B8;
        PresentationParameters.BackBufferCount = b;
        PresentationParameters.MultiSampleType = D3DMULTISAMPLE_NONE;
        PresentationParameters.MultiSampleQuality = 0;
        PresentationParameters.SwapEffect = D3DSWAPEFFECT_FLIP;
        PresentationParameters.hDeviceWindow = hWnd;
        PresentationParameters.Windowed = TRUE;
        PresentationParameters.EnableAutoDepthStencil = FALSE;
        PresentationParameters.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;
        PresentationParameters.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
        PresentationParameters.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

        hRet = pDirect3D9->CreateDevice
        (
            D3DADAPTER_DEFAULT,
            D3DDEVTYPE_HAL,
            hWnd,
            D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED,
            &PresentationParameters,
            &pDirect3DDevice9
        );

        if(SUCCEEDED(hRet)) { break; }
    }

    if(FAILED(hRet))
    {
        return false;
    }

    /*! initialize front palette */
    GeneratePalette(fpcRGB32, pColorTable);

    /*! initialize back palette */
    GeneratePalette(fpcRGB32, pColorTableBack);

    return true;
}

bool CRenderDX9::RenderPixels(uint08 *pPixels)
{
    IDirect3DSurface9 *pSurface9 = NULL;

    HRESULT hRet = pDirect3DDevice9->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurface9);

    if(FAILED(hRet))
    {
        return false;
    }

    D3DLOCKED_RECT LockedRect;

    hRet = pSurface9->LockRect(&LockedRect, NULL, 0);

    if(FAILED(hRet))
    {
        pSurface9->Release();
        return false;
    }

    /*
    static char tempR[256] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,2,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,6,6,6,7,7,8,8,8,9,9,10,10,11,
                        11,12,12,13,13,14,14,15,15,16,17,17,18,18,19,20,20,21,22,22,23,23,24,25,25,26,27,27,28,29,30,30,31,32,32,33,34,34,35,36,36,37,38,38,39,40,40,41,41,42,43,43,44,44,45,46,46,47,47,48,48,49,49,50,
                        50,51,51,52,52,52,53,53,54,54,54,55,55,55,56,56,56,57,57,57,57,58,58,58,58,59,59,59,59,59,60,60,60,60,60,60,60,61,61,61,61,61,61,61,61,61,61,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,
                        62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62,62};

    static char tempG[256] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,
                        3,3,3,3,3,3,4,4,4,4,4,4,5,5,5,5,5,6,6,6,6,7,7,7,7,8,8,8,8,9,9,9,9,10,10,10,11,11,11,12,12,12,13,13,13,14,14,14,15,15,15,16,16,16,17,17,18,18,18,19,19,19,20,20,
                        21,21,21,22,22,23,23,23,24,24,25,25,26,26,26,27,27,28,28,29,29,29,30,30,31,31,32,32,32,33,33,34,34,35,35,35,36,36,37,37,37,38,38,39,39,40,40,40,41,41,42,42,42,43,43,43,44,44,45,45,45,46,46,46,
                        47,47,47,48,48,48,49,49,49,50,50,50,50,51,51,51,52,52,52,52,53,53,53,53,54,54,54,54,55,55,55,55,55,56,56,56,56,56,57,57,57,57,57,57,58,58,58,58,58,58,59,59,59,59,59,59,59,59,60,60,60,60,60,60};

    static char tempB[256] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                        0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                        1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,5,5,
                        5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,10,10,10,10,10,10,10,10,11,11,11,11,11};
    //*/

    uint32 *xrgb = (uint32*)LockedRect.pBits;
    uint32  wadd = (LockedRect.Pitch / 4) - width;

    int x, y, w = width, h = height;

    if(!bStereogram)
    {
        for(y=0;y<h;y++)
        {
            for(x=0;x<w;x++)
            {
                *xrgb++ = pColorTable[*pPixels++];
            }

            xrgb += wadd;
        }
    }
    else
    {
        static float dist = 2.0f;
        static float distS = 0.04f;

        if(dist > 5.0f || dist < 2.0f)
            distS = -distS;

        dist += distS;

        #define round(X) (int)((X)+0.5)
        #define DPI 72
        #define E round(dist*DPI)
        #define mu (1/3.0f)
        #define separation(Z) round((1-mu*Z)*E/(2-mu*Z))

        static uint32 *pix = 0;
        static int *same = 0;
    
        if(pix == 0)
        {
            pix = (uint32*)malloc(w*sizeof(uint32));
        }

        if(same == 0)
        {
            same = (int*)malloc(w*sizeof(int));
        }

        for(y=0;y<h;y++)
        {
            int s, left, right;

            for(x=0;x<w;x++)
            {
                same[x] = x;
            }

            for(x=0;x<w;x++)
            {
                float z = (float)(*pPixels) / 255.0f;

                s = separation(z);

                left = x - s/2;
                right = left + s;

                if(0 <= left && right < w)
                {
                    bool visible = true;

                    if(false)
                    {
                        int t=1;

                        float zt;

                        do
                        {
                            float zmt = (float)(*(pPixels - t)) / 255.0f;
                            float zpt = (float)(*(pPixels + t)) / 255.0f;

                            zt = z + 2*(2 - mu*z)*t/(mu*E);

                            visible = (zmt < zt) && (zpt < zt);

                            t++;
                        }
                        while(visible && zt < 1);
                    }

                    if(visible)
                    {
                        int remLeft = left;
                        int remRight = right;
                        int l = same[left];

                        while(l != left && l != right)
                        {
                            if(l < right)
                            {
                                left = l;
                                l = same[left];
                            }
                            else
                            {
                                same[left] = right;
                                left = right;
                                l = same[left];
                                right = l;
                            }
                        }

                        same[left] = right;
                    }
                }

                pPixels++;
            }

            for(x=0;x<w;x++)
            {
                if(same[x] == x)
                {
                    pix[x] = (rand()%2 == 0) ? 0x00000000 : 0x00FFFFFF;
                    //pix[x] = (rand()%2 == 0) ? 0x00000044 : 0x00FFFFFF;
                    //pix[x] = (rand() | (rand() << 16)) & 0x00FFFFFF;
                }
                else
                {
                    pix[x] = pix[same[x]];
                }

                *xrgb++ = pix[x];
            }

            xrgb += wadd;
        }
    }

    //free(pix);
    //free(same);

    pSurface9->UnlockRect();

    pSurface9->Release();

    pDirect3DDevice9->Present(NULL, NULL, NULL, NULL);

    return true;
}

bool CRenderDX9::RotatePalette()
{
    if(rotPaletteOffset < 256*2)
    {
        //
        // NOTE: Solar Rotation
        //

        //pColorTable[rotPaletteOffset] = pColorTableBack[rotPaletteOffset];
        if(rotPaletteOffset%2 == 0)
        {
            for(int r=0;r<256;r++)
            {
                uint32 rs = pColorTableBack[r] & 0x00FF0000;
                uint32 gs = pColorTableBack[r] & 0x0000FF00;
                uint32 bs = pColorTableBack[r] & 0x000000FF;

                uint32 rd = pColorTable[r] & 0x00FF0000;
                uint32 gd = pColorTable[r] & 0x0000FF00;
                uint32 bd = pColorTable[r] & 0x000000FF;

                if(rd > rs) { rd -= 0x00010000; }
                else if(rd < rs) { rd += 0x00010000; }

                if(gd > gs) { gd -= 0x00000100; }
                else if(gd < gs) { gd += 0x00000100; }

                if(bd > bs) { bd -= 0x00000001; }
                else if(bd < bs) { bd += 0x00000001; }

                pColorTable[r] = rd | gd | bd;
            }
        }
    }
    else if(rotPaletteOffset == 256*2)
    {
        GeneratePalette(fpcRGB32, pColorTableBack);

        rotPaletteOffset = -1;
    }

    rotPaletteOffset++;

    return true;
}

void CRenderDX9::fpcRGB32(void *pContext, uint32 i, float r, float g, float b)
{
    uint32 *pColorTable = (uint32*)pContext;

    /*
    static bool    bCached = false;
    static uint08 *pRGB = NULL;

    if(!bCached)
    {
        FILE *kthx = fopen("C:\\palette.raw", "rb");

        pRGB = (uint08*)malloc(3*sizeof(uint08)*256);

        fread(pRGB, 3*sizeof(uint08)*256, 1, kthx);

        fclose(kthx);
    }
    //*/

    uint08 uiR = (uint08)(r*255.0f);
    uint08 uiG = (uint08)(g*255.0f);
    uint08 uiB = (uint08)(b*255.0f);

    /*
    uiR = pRGB[i*3+0];
    uiG = pRGB[i*3+1];
    uiB = pRGB[i*3+2];
    //*/

    pColorTable[i] = 0xFF000000 | (uiR << 16) | (uiG << 8) | uiB;

    return;
}
