// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef ARRAYLIST_H
#define ARRAYLIST_H

#include "BaseTypes.h"

/*! @brief List object that allows very fast insert/remove/exists functionality. */
class ArrayList
{
    public:

        /*! @note magicKey is the flag value used to mark an array index as empty! */
        ArrayList(int maxSize = 128, int magicKey = 0) : maxSize(maxSize), magicKey(magicKey) 
        { 
            listArray = new int[maxSize];
            
            int r;

            for(r=0;r<maxSize;r++)
            {
                listArray[r] = magicKey;
            }
        }

       ~ArrayList() { delete[] listArray; }

        /*! insert value into list */
        bool insert(int value)
        {
            int r;

            for(r=0;r<maxSize;r++)
            { 
                if(listArray[r] == magicKey) 
                { 
                    listArray[r] = value;
                    return true;
                } 
            }

            return false;
        }

        /*! remove all instances of value from list */
        void remove(int value)
        {
            int r;

            for(r=0;r<maxSize;r++) 
            { 
                if(listArray[r] == value) 
                { 
                    listArray[r] = magicKey; 
                } 
            }
        }

        /*! remove one instance of value from list */
        bool remove_once(int value)
        {
            int r;

            for(r=0;r<maxSize;r++)
            {
                if(listArray[r] == value)
                {
                    listArray[r] = magicKey;
                    return true;
                }
            }

            return false;
        }

        /*! check if a value exists in list */
        bool exists(int value)
        {
            int r;

            for(r=0;r<maxSize;r++) 
            { 
                if(listArray[r] == value) 
                { 
                    return true; 
                } 
            }

            return false;
        }

    private:

        /*! actual array */
        int *listArray;

        /*! maximum size of array */
        int maxSize;

        /*! magic key used to mark an array index as empty */
        int magicKey;

};

#endif

