// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "MutexRW.h"
#include "Thread.h"

MutexRW::MutexRW() : mutexW(), mutexR()
{
    readerCount = 0;

    noReaderEvent.Set();
}

void MutexRW::WaitW()
{
    mutexW.Wait();

    noReaderEvent.Wait();
    noReaderEvent.Set();
}

void MutexRW::WaitR()
{
    int threadID = Thread::GetCurrentThreadID();

    bool exists = readerList.exists(threadID);

    if(!exists) { mutexW.Wait(); }

    mutexR.Wait();
    readerCount++;
    noReaderEvent.Reset();
    mutexR.Release();

    if(!exists) { mutexW.Release(); }

    readerList.insert(threadID);
}

void MutexRW::ReleaseW()
{
    mutexW.Release();
}

void MutexRW::ReleaseR()
{
    int threadID = Thread::GetCurrentThreadID();

    mutexR.Wait();
    readerCount--;
    if(readerCount == 0) { noReaderEvent.Set(); }
    mutexR.Release();

    readerList.remove_once(threadID);
}
