// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef MUTEXRW_H
#define MUTEXRW_H

#include "BaseTypes.h"

#include "Mutex.h"
#include "Event.h"
#include "ArrayList.h"

/*! @brief Mutex object that allows multiple readers and a single writer */
class MutexRW
{
    public:

        MutexRW();

        /*! Wait for writer mutex ownership */
        void WaitW();

        /*! Wait for reader mutex ownership */
        void WaitR();

        /*! Release writer mutex ownership */
        void ReleaseW();

        /*! Release reader mutex ownership */
        void ReleaseR();

        /*! Switch from reader to writer @note this operation is not atomic! */
        void SwitchRtoW() { ReleaseR(); WaitW(); }
        
        /*! Switch from writer to reader @note this operation is not atomic! */
        void SwitchWtoR() { ReleaseW(); WaitR(); }

    private:

        /*! list of current readers */
        ArrayList readerList;

        /*! number of current readers */
        int readerCount;

        /*! mutex for writer */
        Mutex mutexW;

        /*! mutex for reader */
        Mutex mutexR;

        /*! event for when readerCount := 0 */
        Event noReaderEvent;
};

#endif
