// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "RenderWnd.h"

#include "resource.h"

RenderWnd::RenderWnd(uint width, uint height, bool bPopup)
{
    this->mouseX = width / 2;
    this->mouseY = height / 2;

    this->timerID = 0;
    this->curTick = 0;

    this->mouseDownL = false;

    this->isMaximized = false;

    this->szClassName = "RenderWnd";
    this->szWindowName = "Render Window";

    if(bPopup) { this->dwWindowStyle = WS_POPUP; }

    this->uiClassStyle |= CS_DBLCLKS;

    this->width = width;
    this->height = height;

    // center to desktop
    {
        RECT Rect;

        GetWindowRect(GetDesktopWindow(), &Rect);

        this->x = Rect.left + (Rect.right - Rect.left)/2 - this->width/2;
        this->y = Rect.top + (Rect.bottom - Rect.top)/2 - this->height/2;
    }

    return;
}

RenderWnd::~RenderWnd()
{
}

LRESULT CALLBACK RenderWnd::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_CREATE:
        {
            // resize window so that client area := widthxheight
            {
                RECT cRect;
                RECT wRect;

                GetClientRect(hWnd, &cRect);
                GetWindowRect(hWnd, &wRect);

                uint32 difW = (wRect.right  - wRect.left) - (cRect.right);
                uint32 difH = (wRect.bottom - wRect.top)  - (cRect.bottom);

                this->width += difW;
                this->height += difH;

                MoveWindow(hWnd, wRect.left, wRect.top, this->width, this->height, TRUE);
            }

            curTick = 0;

            timerID = SetTimer(hWnd, 1, 500, 0);
        }
        break;

        case WM_DESTROY:
            KillTimer(hWnd, timerID);
            break;

        case WM_TIMER:
            if(!this->isMaximized) { curTick = 0; }
            if(curTick++ == 4) { SetCursor(NULL); }
            break;

        case WM_SYSKEYDOWN:
        {
            switch(wParam)
            {
                case VK_RETURN:
                    SendMessage(hWnd, WM_LBUTTONDBLCLK, 0, 0);
                    break;
            }
        }
        break;

		case WM_KEYDOWN:
        {
			switch (wParam)
			{
				case VK_ESCAPE:
                    if(this->isMaximized) { SendMessage(hWnd, WM_LBUTTONDBLCLK, 0, 0); }
					break;
			}
        }
        break;

        case WM_MOUSEMOVE:
        {
            curTick = 0;

            this->mouseX = LOWORD(lParam);
            this->mouseY = HIWORD(lParam);
        }
        break;

        case WM_SIZE:
        {
            RECT Rect;

            GetClientRect(hWnd, &Rect);

            this->width  = Rect.right;
            this->height = Rect.bottom;
        }
        break;

        case WM_LBUTTONDOWN:
            this->mouseDownL = true;
            break;

        case WM_LBUTTONUP:
            this->mouseDownL = false;
            break;

        case WM_LBUTTONDBLCLK:
        {
            if(this->isMaximized)
            {
                int x = this->cachedRect.left;
                int y = this->cachedRect.top;
                int cx = this->cachedRect.right - this->cachedRect.left;
                int cy = this->cachedRect.bottom - this->cachedRect.top;

                SetWindowLong(hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
                SetWindowPos(hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
                ShowWindow(hWnd, SW_RESTORE);
                SetWindowPos(hWnd, HWND_NOTOPMOST, x, y, cx, cy, 0);

                this->isMaximized = false;
            }
            else
            {
                GetWindowRect(hWnd, &this->cachedRect);
                SetWindowLong(hWnd, GWL_STYLE, WS_POPUP);
                ShowWindow(hWnd, SW_MAXIMIZE);
                SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
                SetFocus(hWnd);

                this->isMaximized = true;
            }
        }
        break;
    }

    return Wnd::WndProc(hWnd, uMsg, wParam, lParam);
}
