// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "Thread.h"

#ifdef _XBOX
#include <xtl.h>
#else
#include <windows.h>
#endif

static DWORD WINAPI ThreadFuncFwd(LPVOID lpThreadParameter)
{
    Thread *pThis = (Thread*)lpThreadParameter;

    pThis->GetFunc()(pThis->GetStopFlagRef(), pThis->GetThreadContext());

    SetEvent(pThis->GetStoppedEvent());

    return 0;
}

Thread::Thread(ThreadFunc pfFunc, void *pContext) : pfFunc(pfFunc), pContext(pContext), pHandle(NULL), bStop(false)
{
    this->pStoppedEvent = (void*)CreateEvent(NULL, TRUE, TRUE, NULL);
}

Thread::~Thread()
{
    this->Stop();

    if(pStoppedEvent != NULL) { CloseHandle(this->pStoppedEvent); }
}

void Thread::Start()
{
    DWORD dwThreadId;

    ResetEvent(this->pStoppedEvent);

    this->bStop = false;

    this->pHandle = (void*)CreateThread(NULL, NULL, ThreadFuncFwd, this, NULL, &dwThreadId);
}

void Thread::Stop(bool async)
{
    if(this->pHandle == NULL) { return; }

    this->bStop = true;

    if(!async) { WaitForSingleObject(this->pStoppedEvent, INFINITE); }

    pHandle = NULL;
}

bool Thread::IsStopped()
{
    DWORD dwRet = WaitForSingleObject(this->pStoppedEvent, 0);

    return (dwRet == WAIT_OBJECT_0) || (dwRet == WAIT_ABANDONED);
}

void Thread::Sleep(int ms)
{
    ::Sleep(ms);
    return;
}

int Thread::GetCurrentThreadID()
{
    return (int)::GetCurrentThreadId();
}
