// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "Wnd.h"

Wnd::Wnd()
{
    this->szClassName = "SirenicWnd";
    this->szWindowName = "Sirenic Generic Window";
    this->x = 150;
    this->y = 150;
    this->width = 320;
    this->height = 240;
    this->hParentWnd = NULL;
    this->uiClassStyle = 0;
    this->dwWindowStyle = WS_OVERLAPPEDWINDOW;
    this->hIcon = 0;
    this->hBgBrush = (HBRUSH)GetStockObject(BLACK_BRUSH);
    this->bCreated = false;

    return;
}

Wnd::~Wnd()
{
    UnregisterClass(this->szClassName, GetModuleHandle(NULL));
}

bool Wnd::Create()
{
    if(this->bCreated)
        return true;

    WNDCLASS WndClass;

    WndClass.hInstance     = GetModuleHandle(NULL);
    WndClass.lpszClassName = this->szClassName;
    WndClass.lpfnWndProc   = Wnd::WndProcForward;
    WndClass.style         = this->uiClassStyle;
    WndClass.hIcon         = this->hIcon;
    WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
    WndClass.lpszMenuName  = NULL;
    WndClass.cbClsExtra    = 0;
    WndClass.cbWndExtra    = 0;
    WndClass.hbrBackground = this->hBgBrush;

    this->wClass = RegisterClass(&WndClass);
    
    this->hWnd = CreateWindowEx
    (
        NULL,
        this->szClassName,
        this->szWindowName,
        this->dwWindowStyle,
        this->x == 0 ? CW_USEDEFAULT : this->x,
        this->y == 0 ? CW_USEDEFAULT : this->y,
        this->width,
        this->height,
        this->hParentWnd,
        NULL,
        GetModuleHandle(NULL),
        this
    );

    if(this->hWnd == 0)
    {
        UnregisterClass(this->szClassName, GetModuleHandle(NULL));
        return false;
    }

    SetWindowLong(this->hWnd, GWL_USERDATA, (LONG)this);

    this->bCreated = true;

    return true;
}

bool Wnd::ProcessMessages()
{
    if(!this->bCreated)
        return false;

    MSG msg;

    if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
    {
        if(!GetMessage(&msg, NULL, 0, 0))
        {
            UnregisterClass(this->szClassName, GetModuleHandle(NULL));
            return false;
        }

        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return true;
}

LRESULT CALLBACK Wnd::WndProcForward(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    Wnd *pForward = (Wnd*)GetWindowLong(hWnd, GWL_USERDATA);

    if(pForward == 0)
    {
        if(uMsg == WM_CREATE)
        {
            LPCREATESTRUCT cs = (LPCREATESTRUCT)lParam;

            pForward = (Wnd*)cs->lpCreateParams;
        }
        else
        {
            return DefWindowProc(hWnd, uMsg, wParam, lParam);
        }
    }
 
    return pForward->WndProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK Wnd::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
	{
        case WM_MOVING:
        {
            RECT *pRect = (RECT*)lParam;

            if(pRect != NULL)
            {
                this->x = pRect->left;
                this->y = pRect->top;

                this->width = pRect->right - pRect->left;
                this->height = pRect->bottom - pRect->top;
            }
        }
        break;

        case WM_CLOSE:
            DestroyWindow(hWnd);
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
			break;

        default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

    return 0;
}
