// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_Bezier.h"
#include "Sirenic.h"

#include <math.h>

FX_Bezier::FX_Bezier(Sirenic *pSirenic, uint08 color, CartesianTransformFunc pFuncX, CartesianTransformFunc pFuncY) : RenderFX(pSirenic), color(color), pFuncX(pFuncX), pFuncY(pFuncY)
{
    int v;

    for(v=0;v<4;v++) { fT[v] = -1; }
}

bool FX_Bezier::Render()
{
    float fW = (float)pSirenic->GetWidth();
    float fH = (float)pSirenic->GetHeight();

    CartCoord fCoord[4];

    int v;

    for(v=0;v<4;v++)
    {
        pFuncX(fT[v], fCoord[v].fX);
        pFuncY(fT[v], fCoord[v].fY);

        fCoord[v].fX *= fW;
        fCoord[v].fY *= fH;
    }

    pSirenic->DrawBezier(fCoord, 0.01f, 255);

    return true;
}

bool FX_Bezier::Step()
{
    fT[0] += 0.0200f;
    fT[1] += 0.0175f;
    fT[2] += 0.0150f;
    fT[3] += 0.0125f;

    if(fT[0] > 1.0f) { fT[0] -= 2; }
    if(fT[1] > 1.0f) { fT[1] -= 2; }
    if(fT[2] > 1.0f) { fT[2] -= 2; }
    if(fT[3] > 1.0f) { fT[3] -= 2; }

    return true;
}
