// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef FX_BEZIER_H
#define FX_BEZIER_H

#include "RenderFX.h"
#include "Transform.h"

/*! @brief Bezier curve rendering effect */
class FX_Bezier : public RenderFX
{
    public:

        /*! Construct a bezier curve rendering effect using the specified color and transformation functions */
        FX_Bezier(Sirenic *pSirenic, uint08 color, CartesianTransformFunc pFuncX = CTF_SPIN_X, CartesianTransformFunc pFuncY = CTF_SPIN_Y);

        /*! Perform rendering */
        bool Render();

        /*! Increment state machine one step */
        bool Step();

    private:

        /*! Cache of cartesian transformation functions */
        CartesianTransformFunc pFuncX;
        CartesianTransformFunc pFuncY;

        /*! Offset for Bezier coordinate sine functions */
        float fT[4];

        /*! Color to use for rendering */
        uint08 color;
};

#endif
