// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_CMachineWorld.h"
#include "Sirenic.h"

#include <stdlib.h>
#include <math.h>

FX_CMachineWorld::FX_CMachineWorld(Sirenic *pSirenic, uint08 color) : RenderFX(pSirenic), CMachineWorld(), color(color)
{
    pClosestNode = 0;
    fClosestLastX = fClosestLastY = 0;

    iGravSkipper = 0;
}

bool FX_CMachineWorld::Render()
{
    float fW = (float)pSirenic->GetWidth();
    float fH = (float)pSirenic->GetHeight();

    /*! iterate through all machines */
    for(std::list<CMachine *>::iterator p = MachineList.begin(); p != MachineList.end(); ++p)
    {
        CMachineNodeList *pCurNode = (*p)->GetNodeList();

        /*! iterate through all nodes in current machine */
        while(pCurNode != 0)
        {
            float fPosX = pCurNode->GetPosX();
            float fPosY = pCurNode->GetPosY();

            int fX0 = (int)(fPosX*fW);
            int fY0 = (int)((1.0f - fPosY)*fH);

            /*! highlight the node closest to mouse coordinates */
            if(pCurNode == pClosestNode)
            {
                pSirenic->DrawRect(fX0-4, fY0-4, fX0+4, fY0+4, color);
            }

            /*! draw current node */
            pSirenic->DrawPoint(fX0-1, fY0-1, color);
            pSirenic->DrawPoint(fX0  , fY0-1, color);
            pSirenic->DrawPoint(fX0+1, fY0-1, color);
            pSirenic->DrawPoint(fX0-1, fY0,   color);
            pSirenic->DrawPoint(fX0  , fY0,   color);
            pSirenic->DrawPoint(fX0+1, fY0,   color);
            pSirenic->DrawPoint(fX0-1, fY0+1, color);
            pSirenic->DrawPoint(fX0  , fY0+1, color);
            pSirenic->DrawPoint(fX0+1, fY0+1, color);

            /*! advance to next node */
            pCurNode = pCurNode->pNext;
        }

        /*! iterate through all connections in current machine */
        CMachineConnList *pCurConn = (*p)->GetConnList();

        while(pCurConn != 0)
        {
            float fPosX0 = pCurConn->GetNode0()->GetPosX();
            float fPosY0 = pCurConn->GetNode0()->GetPosY();

            float fPosX1 = pCurConn->GetNode1()->GetPosX();
            float fPosY1 = pCurConn->GetNode1()->GetPosY();

            int fX0 = (int)(fPosX0*fW);
            int fY0 = (int)((1.0f - fPosY0)*fH);

            int fX1 = (int)(fPosX1*fW);
            int fY1 = (int)((1.0f - fPosY1)*fH);

            /*! draw current connection */
            pSirenic->DrawLineWu(fX0, fY0, fX1, fY1, (float)color / 255.0f);

            /*! advance to next connection */
            pCurConn = pCurConn->pNext;
        }
    }

    return true;
}

bool FX_CMachineWorld::Step()
{
    /*! determine mouse coordinates */
    float fMouseX = (float)pSirenic->GetMousePosX();
    float fMouseY = (float)pSirenic->GetMousePosY();

    fMouseX /= pSirenic->GetWidth();
    fMouseY /= pSirenic->GetHeight();

    fMouseY = 1.0f - fMouseY;

    if(pSirenic->GetMouseDownL())
    {
        if(pClosestNode != 0)
        {
            float fPosX = pClosestNode->GetPosX();
            float fPosY = pClosestNode->GetPosY();

            pClosestNode->SetPos(fMouseX, fMouseY);
            pClosestNode->SetPoss(0, 0);

            fClosestLastX = fPosX;
            fClosestLastY = fPosY;
        }
    }
    else
    {
        /*! release last node */
        /*
        if(pClosestNode != 0)
        {
            float fPosX = pClosestNode->GetPosX();
            float fPosY = pClosestNode->GetPosY();

            pClosestNode->SetPoss(fClosestLastX - fPosX, fClosestLastY - fPosY);
        }
        //*/

        /*! locate nearest node */
        CMachineNodeList *pNewClosestNode = 0;

        for(std::list<CMachine *>::iterator p = MachineList.begin(); p != MachineList.end(); ++p)
        {
            CMachineNodeList *pCurNode = (*p)->GetNodeList();

            float fClosestDist = 100.0f;

            while(pCurNode != 0)
            {
                float fPosX = pCurNode->GetPosX();
                float fPosY = pCurNode->GetPosY();

                float fPosXs = pCurNode->GetPosXs();
                float fPosYs = pCurNode->GetPosYs();

                float fDiffX = fMouseX - fPosX;
                float fDiffY = fMouseY - fPosY;

                float fDistance = sqrtf(fDiffX*fDiffX + fDiffY*fDiffY);

                if(fDistance < 0.02f && (fDistance < fClosestDist))
                {
                    fClosestDist = fDistance;
                    pNewClosestNode = pCurNode;
                }

                /*! advance to next node */
                pCurNode = pCurNode->pNext;
            }
        }

        pClosestNode = pNewClosestNode;

        if(pClosestNode != 0)
        {
            fClosestLastX = pClosestNode->GetPosX();
            fClosestLastY = pClosestNode->GetPosY();
        }
    }

    SetGravity(0, -0.0005f);

    /* apply random gravity
    if(iGravSkipper++%20 == 0)
    {
        float fGravX = ((float)rand() / RAND_MAX) * 0.0010f - 0.0005f;
        float fGravY = ((float)rand() / RAND_MAX) * 0.0010f - 0.0005f;

        SetGravity(fGravX, fGravY);
    }
    //*/

    IncrementTime();

    return true;
}
