// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_PolarThreader.h"
#include "Sirenic.h"

#include <math.h>

FX_PolarThreader::FX_PolarThreader(Sirenic *pSirenic, float fSize, uint08 color, float posXs, float posYs, bool trail) : RenderFX(pSirenic), fSize(fSize), color(color), posXs(posXs), posYs(posYs), trail(trail)
{
    float fW = (float)pSirenic->GetWidth();
    float fH = (float)pSirenic->GetHeight();

    posX = ((float)rand() / (float)RAND_MAX) * 3.14159265f * 2 - 3.14159265f;
    posY = ((float)rand() / (float)RAND_MAX) * 3.14159265f * 2 - 3.14159265f;

    lastRX = rX = (sinf(posX)/(1.0f/fSize))*(fW/2) + fW/2;
    lastRY = rY = (cosf(posY)/(1.0f/fSize))*(fH/2) + fH/2;
}

bool FX_PolarThreader::Render()
{
    if(trail)
    {
        pSirenic->DrawLine((int)lastRX, (int)lastRY, (int)rX, (int)rY, color);
        pSirenic->DrawLine((int)lastRX, (int)lastRY - 1, (int)rX, (int)rY - 1, color);
        pSirenic->DrawLine((int)lastRX, (int)lastRY + 1, (int)rX, (int)rY + 1, color);
    }
    else
    {
        pSirenic->DrawRect((int)rX-1, (int)rY-1, (int)rX+1, (int)rY+1, color);
    }

    return true;
}

bool FX_PolarThreader::Step()
{
    float w = (float)pSirenic->GetWidth();
    float h = (float)pSirenic->GetHeight();

    lastRX = rX;
    lastRY = rY;

    rX = (sinf(posX)/(1.0f/fSize))*(w/2) + w/2;
    rY = (cosf(posY)/(1.0f/fSize))*(h/2) + h/2;

    posX += posXs;
    posY += posYs;

    return true;
}
