// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef FX_POLARTHREADER_H
#define FX_POLARTHREADER_H

#include "RenderFX.h"

/*! @brief Polar threader rendering effect */
class FX_PolarThreader : public RenderFX
{
    public:

        /*! 
         *  Construct polar threader effect using the specified configuration
         *
         *  @param fSize diameter of flying pattern
         *  @param color color to render effect with
         *  @param posXs beginning X component of acceleration
         *  @param posYs beginning Y component of acceleration
         *  @param trail turns "trail" rendering on (default is off)
         *
         */

        FX_PolarThreader(Sirenic *pSirenic, float fSize, uint08 color, float posXs = 0.035f, float posYs = 0.025f, bool trail = false);

        /*! Perform rendering */
        bool Render();

        /*! Increment state machine one step */
        bool Step();

    private:

        /*! current position of threader */
        float posX,  posY;

        /*! current acceleration of threader */
        float posXs, posYs;

        /*! current fully transformed position of threader */
        float rX, rY;

        /*! previous fully transformed position of threader */
        float lastRX, lastRY;

        /*! diameter of flying pattern */
        float fSize;

        /*! color to render effect with */
        uint08 color;

        /*! should we render a "trail" blur effect? */
        bool trail;
};

#endif
