// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_PolyDance.h"
#include "Sirenic.h"

#include <math.h>

FX_PolyDance::FX_PolyDance(Sirenic *pSirenic, float poss, float scaleX, float scaleXs, float scaleY, float scaleYs, float c, float cs) 
 : RenderFX(pSirenic),
   pos(0),
   poss(poss),
   cx(0),
   cy(1),
   c(c),
   cs(cs),
   scaleX(scaleX),
   scaleY(scaleY),
   scaleXs(scaleXs),
   scaleYs(scaleYs)
{

}

bool FX_PolyDance::Render()
{
    float fW = (float)pSirenic->GetWidth();
    float fH = (float)pSirenic->GetHeight();

    float x1 = sinf(pos)*(fW/2)*scaleX + fW/2;
    float y1 = sinf(pos)*(fH/2)*scaleY + fH/2;

    float x2 = cosf(pos+cx)*(fW/2)*scaleX + fW/2;
    float y2 = cosf(pos+cy)*(fH/2)*scaleY + fH/2;

    /*! draw roaming polygon */
    //*
    pSirenic->DrawLine((int)x1, (int)y1, (int)x2, (int)y2, (uint08)(c*255.0f));
    pSirenic->DrawLine((int)x2, (int)y2, (int)(fW - x1), (int)(fH - y1), (uint08)(c*255.0f));
    pSirenic->DrawLine((int)(fW - x1), (int)(fH - y1), (int)(fW - x2), (int)(fH - y2), (uint08)(c*255.0f));
    pSirenic->DrawLine((int)(fW - x2), (int)(fH - y2), (int)x1, (int)y1, (uint08)(c*255.0f));
    //*/
    /*
    pSirenic->DrawLineWu(x1, y1, x2, y2, c);
    pSirenic->DrawLineWu(x2, y2, fW - x1, fH - y1, c);
    pSirenic->DrawLineWu(fW - x1, fH - y1, fW - x2, fH - y2, c);
    pSirenic->DrawLineWu(fW - x2, fH - y2, x1, y1, c);
    //*/

    return true;
}

bool FX_PolyDance::Step()
{
    pos += poss;

    cx += 0.004f;
    cy += 0.005f;

    c = c + cs;

    if(c > 1.0f)
    {
        c = 1.0f;
        cs = -cs;
    }
    else if(c < 0.0f)
    {
        c = 0.0f;
        cs = -cs;
    }

    if(scaleX >= 0.5 || scaleX <= 0)
        scaleXs = - scaleXs;

    if(scaleY >= 0.5 || scaleY <= 0)
        scaleYs = - scaleYs;

    scaleY += scaleYs;
    scaleX += scaleXs;

    return true;
}
