// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_RandomSquares.h"
#include "Sirenic.h"

#include <math.h>

FX_RandomSquares::FX_RandomSquares(Sirenic *pSirenic, float minSizeX, float minSizeY, float maxSizeX, float maxSizeY, float minColor, float maxColor, int count)
 : RenderFX(pSirenic),
   minSizeX(minSizeX),
   minSizeY(minSizeY),
   maxSizeX(maxSizeX),
   maxSizeY(maxSizeY),
   difSizeX(maxSizeX - minSizeX),
   difSizeY(maxSizeY - minSizeY),
   minColor(minColor),
   maxColor(maxColor),
   difColor(maxColor - minColor),
   count(count)
{
}

bool FX_RandomSquares::Render()
{
    float fW = (float)pSirenic->GetWidth();
    float fH = (float)pSirenic->GetHeight();

    int v;

    /*! draw the specified number of squares */
    for(v=0;v<this->count;v++)
    {
        /*! randomize color */
        uint08 c = (char)((minColor + difColor*((float)rand() / (float)RAND_MAX))*255.0f);

        /*! randomize center position */
        float posX = (float)rand() / (float)RAND_MAX;
        float posY = (float)rand() / (float)RAND_MAX;

        /*! randomize size */
        float sx = (minSizeX + difSizeX*((float)rand() / (float)RAND_MAX))*fW;
        float sy = (minSizeY + difSizeY*((float)rand() / (float)RAND_MAX))*fH;

        /*! calculate square corner positions */
        float x0 = posX*fW - (sx/2.0f);
        float y0 = posY*fH - (sy/2.0f);
        float x1 = x0 + sx;
        float y1 = y0 + sy;

        /*! draw current square */
        pSirenic->DrawRect((int)x0, (int)y0, (int)x1, (int)y1, c);
    }

    return true;
}

bool FX_RandomSquares::Step()
{
    return true;
}
