// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef FX_RANDOMSQUARES_H
#define FX_RANDOMSQUARES_H

#include "RenderFX.h"

/*! @brief Random squares rendering effect */
class FX_RandomSquares : public RenderFX
{
    public:

        /*! 
         *  Construct random squares effect using the specified configuration
         *
         *  @param minSizeX minimum X dimension of a square
         *  @param minSizeY minimum Y dimension of a square
         *  @param maxSizeX maximum X dimension of a square
         *  @param maxSizeY maximum Y dimension of a square
         *  @param minColor minimum color value of a square
         *  @param maxColor maximum color value of a square
         *  @param count number of squares to render per frame
         *
         */

        FX_RandomSquares(Sirenic *pSirenic, float minSizeX, float minSizeY, float maxSizeX, float maxSizeY, float minColor, float maxColor, int count);

        /*! Perform rendering */
        bool Render();

        /*! Increment state machine one step */
        bool Step();

    private:

        /*! cached minimum size of square */
        float minSizeX, minSizeY;

        /*! cached maximum size of square */
        float maxSizeX, maxSizeY;

        /*! cached difference between min and max size of square */
        float difSizeX, difSizeY;

        /*! cached minimum/maximum color value of square */
        float minColor, maxColor;

        /*! cached difference between min and max color value of square */
        float difColor;

        /*! cached number of squares to render per frame */
        int count;
};

#endif
