// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "FX_TreeFractal.h"
#include "Sirenic.h"

#include <math.h>

FX_TreeFractal::FX_TreeFractal(Sirenic *pSirenic)
 : RenderFX(pSirenic)
{
    fT = 0.0f;
}

bool FX_TreeFractal::Render()
{
    float w = (float)pSirenic->GetWidth();
    float h = (float)pSirenic->GetHeight();

    float cx = w/2;
    float cy = h/2;

    //RecursiveFractal(fT + 0.75, 75.0f, 1.0f, 0.1f, cx, cy, 10);
    //RecursiveFractal(fT - 0.75, 75.0f, 1.0f, 0.1f, cx, cy, 10);

    float minDim = (w > h) ? h : w;

    float fLength = fabsf(cosf(fT/10)*(minDim/8));

    RecursiveFractal(fT, fLength, 0.0f, -0.1f, (uint32)cx, (uint32)cy, 10);

    fT += 0.04f;

    return true;
}

bool FX_TreeFractal::Step()
{
    return true;
}

void FX_TreeFractal::RecursiveFractal(float fRecT, float fLength, float fColor, float fColorDecay, uint32 lastX, uint32 lastY, uint32 ttl)
{
    if(--ttl == 0)
        return;

    float w = (float)pSirenic->GetWidth();
    float h = (float)pSirenic->GetHeight();

    float cx = w/2;
    float cy = h/2;

    float ax = sinf(fRecT)*fLength;
    float ay = cosf(fRecT)*fLength;

    uint32 curX = lastX + (uint32)ax; 
    uint32 curY = lastY + (uint32)ay;

    fLength *= 0.70f;

    pSirenic->DrawLine(lastX, lastY, curX, curY, (uint08)(fColor*255.0f));

    if(fColor - fColorDecay > 0)
    {
    fColor -= fColorDecay;
    }
    else
    {
        fColor = 0;
    }

    //RecursiveFractal(fRecT + 0.75, fLength, fColor, fColorDecay, curX, curY, ttl);
    //RecursiveFractal(fRecT - 0.75, fLength, fColor, fColorDecay, curX, curY, ttl);
    RecursiveFractal(fRecT + sinf(fRecT)*0.75f, fLength, fColor, fColorDecay, curX, curY, ttl);
    RecursiveFractal(fRecT - cosf(fRecT)*0.75f, fLength, fColor, fColorDecay, curX, curY, ttl);

    return;
}
