// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "Sirenic.h"

#include <math.h>

#ifndef _XBOX /*! @todo support DrawString on Xbox */
#include <ft2build.h>
#include <freetype/freetype.h>
#endif

void Sirenic::roDrawLine(uint08 *pPixels, uint uiPitch, uint x0, uint y0, uint x1, uint y1, uint08 c, roAssignFunc assignFunc)
{
    bool bYMax = false;

    int inc;

    int minLen = y1 - y0;
    int maxLen = x1 - x0;

    if(abs(minLen) > abs(maxLen))
    {
        int swap = minLen;

        minLen = maxLen;

        maxLen = swap;
        
        bYMax = true;
    }

    inc = (maxLen < 0) ? -1 : 1;

    float divDiff;

    if(minLen==0) { divDiff = (float)maxLen; }
    else { divDiff=(float)maxLen/(float)minLen; }

    if(bYMax)
    {
        for(int i=0;i!=maxLen;i+=inc)
        {
            uint08 *cur = &pPixels[x0 + (int)((float)i/divDiff) + (y0+i)*uiPitch];

            assignFunc(cur, c);
        }
    }
    else
    {
        for(int i=0;i!=maxLen;i+=inc)
        {
            uint08 *cur = &pPixels[x0 + i + (y0+(int)((float)i/divDiff))*uiPitch];

            assignFunc(cur, c);
        }
    }
}

static inline void fswap(float &a, float &b)
{
    float t = a;

    a = b;
    b = t;

    return;
}

static inline float trunc(float a)
{
    return (float)(int)a;
}

static inline float frac(float a)
{
    return a - trunc(a);
}

static inline float invfrac(float a)
{
    return 1 - frac(a);
}

void Sirenic::roDrawLineWu(uint08 *pPixels, uint uiPitch, float x0, float y0, float x1, float y1, float c)
{
    float grad, xd, yd;
    float xgap, ygap, xend, yend, xf, yf;
    float brightness1, brightness2;

    int x, y, ix0, ix1, iy0, iy1;

    uint08 c1, c2;

    // width / height of line
    xd = (x1 - x0);
    yd = (y1 - y0);

    if(fabs(xd) > fabs(yd))
    {
        if(x0 > x1)
        {
            fswap(x0, x1);
            fswap(y0, y1);

            xd = (x1-x0);
            yd = (y1-y0);
        }

        grad = yd/xd;

        //
        // end point 1
        //

        xend = trunc(x0+0.5f);                      // find nearest integer X-coordinate
        yend = y0 + grad*(xend-x0);                 // and corresponding Y value
                                                    
        xgap = invfrac(x0+0.5f);                    // distance i
                                                    
        ix0  = (int)xend;                           // calc screen coordinates
        iy0  = (int)yend;                           
                                                    
        brightness1 = invfrac(yend) * xgap;         // calc the intensity of the other 
        brightness2 =    frac(yend) * xgap;         // end point pixel pair.
                                                    
        c1 = (uint08)(brightness1 * c * 255);       // calc pixel values
        c2 = (uint08)(brightness2 * c * 255);           

        roAssignAdd(&pPixels[ix0 + iy0*uiPitch], c1);  // draw the pair of pixels
        roAssignAdd(&pPixels[ix0 + iy0*uiPitch + 1], c1);
                                                    
        yf = yend+grad;                             // calc first Y-intersection for
                                                    
        //                                          
        // end point 2                              
        //                                          
                                                    
        xend = trunc(x1+0.5f);                      // find nearest integer X-coordinate
        yend = y1 + grad*(xend-x1);                 // and corresponding Y value
                                                    
        xgap = invfrac(x1-0.5f);                    // distance i
                                                    
        ix1  = (int)xend;                           // calc screen coordinates
        iy1  = (int)yend;                           
                                                    
        brightness1 = invfrac(yend) * xgap;         // calc the intensity of the first 
        brightness2 =    frac(yend) * xgap;         // end point pixel pair.
                                                    
        c1 = (uint08)(brightness1 * c * 255.0f);    // calc pixel values
        c2 = (uint08)(brightness2 * c * 255.0f);           

        roAssignAdd(&pPixels[ix1 + iy1*uiPitch], c1);  // draw the pair of pixels
        roAssignAdd(&pPixels[ix1 + iy1*uiPitch + 1], c1);

        //
        // main loop
        //

        for(x=ix0+1;x<ix1;x++)
        {
            brightness1 = invfrac(yf);              // calc pixel brightness
            brightness2 =    frac(yf);

            c1 = (uint08)(brightness1 * c * 255.0f);// calc pixel values
            c2 = (uint08)(brightness2 * c * 255.0f);

            roAssignAdd(&pPixels[x + ((int)yf)*uiPitch], c1);
            roAssignAdd(&pPixels[x + ((int)yf+1)*uiPitch], c2);

            yf += grad;                             // update the y-coordinate
        }
    }
    else
    {
        if(y0 > y1)
        {
            fswap(x0, x1);
            fswap(y0, y1);

            xd = (x1-x0);
            yd = (y1-y0);
        }

        grad = xd/yd;

        //
        // end point 1
        //

        yend = trunc(y0+0.5f);                      // find nearest integer Y-coordinate
        xend = x0 + grad*(yend-y0);                 // and corresponding X value

        ygap = invfrac(y0+0.5f);                    // distance i

        ix0  = (int)xend;                           // calc screen coordinates
        iy0  = (int)yend;                           

        brightness1 = invfrac(xend) * ygap;         // calc the intensity of the other 
        brightness2 =    frac(xend) * ygap;         // end point pixel pair.

        c1 = (uint08)(brightness1 * c * 255.0f);    // calc pixel values
        c2 = (uint08)(brightness2 * c * 255.0f);           

        roAssignAdd(&pPixels[ix0 + iy0*uiPitch], c1);
        roAssignAdd(&pPixels[ix0 + (iy0 + 1)*uiPitch], c2);        

        xf = xend+grad;                             // calc first X-intersection for

        //
        // end point 2                              
        //

        yend = trunc(y1+0.5f);                      // find nearest integer Y-coordinate
        xend = x1 + grad*(yend-y1);                 // and corresponding X value

        ygap = invfrac(y1-0.5f);                    // distance i

        ix1  = (int)xend;                           // calc screen coordinates
        iy1  = (int)yend;                           

        brightness1 = invfrac(xend) * ygap;         // calc the intensity of the first 
        brightness2 =    frac(xend) * ygap;         // end point pixel pair.

        c1 = (uint08)(brightness1 * c * 255.0f);    // calc pixel values
        c2 = (uint08)(brightness2 * c * 255.0f);           

        roAssignAdd(&pPixels[ix1 + iy1*uiPitch], c1);
        roAssignAdd(&pPixels[ix1 + (iy1 + 1)*uiPitch], c2);

        //
        // main loop
        //

        for(y=iy0+1;y<iy1;y++)
        {
            brightness1 = invfrac(xf);              // calc pixel brightness
            brightness2 =    frac(xf);

            c1 = (uint08)(brightness1 * c * 255.0f);// calc pixel values
            c2 = (uint08)(brightness2 * c * 255.0f);

            roAssignAdd(&pPixels[(int)xf + (y)*uiPitch], c1);
            roAssignAdd(&pPixels[(int)xf + (y+1)*uiPitch], c2);

            xf += grad;                             // update the y-coordinate
        }
    }

    return;
}

void Sirenic::roFillRect(uint08 *pPixels, uint uiPitch, uint x0, uint y0, uint x1, uint y1, uint08 c)
{
    int yStart = y0 > y1 ? y1 : y0;
    int yEnd = y0 > y1 ? y0 : y1;

    int xStart = x0 > x1 ? x1 : x0;
    int xEnd = x0 > x1 ? x0 : x1;

    int lineLength = xEnd - xStart;

    pPixels += yStart*uiPitch + xStart;

    for(int y=yStart;y<yEnd;y++)
    {
        memset(pPixels, c, lineLength);

        pPixels += uiPitch;
    }
}

static inline int getBit(uint08 *pBuffer, uint bit)
{
    int v = (bit / 8);
    int m = (bit % 8);

    return (pBuffer[v] & (0x80 >> m)) >> (7-m);
}

#ifndef _XBOX /*! @todo support DrawString on Xbox */
void Sirenic::roDrawString(uint08 *pBuffer, uint uiPitch, uint uiHeight, uint x0, uint y0, const char *str, int size, bool antiAlias, uint08 c)
{
    static FT_Library ftLibrary;
    static FT_Face    ftFace;

    static bool init = false;

    if(!init)
    {
        int ret = FT_Init_FreeType(&ftLibrary);

        if(ret != 0) { return; }

        ret = FT_New_Face(ftLibrary, "C:\\WINDOWS\\Fonts\\Verdana.ttf", 0, &ftFace);

        if(ret != 0) { return; }

        ret = FT_Set_Char_Size(ftFace, 0, 16*64, 300, 300);

        if(ret != 0) { return; }

        ret = FT_Set_Pixel_Sizes(ftFace, 0, size);

        if(ret != 0) { return; }

        init = true;
    }

    FT_GlyphSlot slot = ftFace->glyph;

    int len = strlen(str);

    for(int n=0;n<len;n++)
    {
        FT_UInt glyph_index;

        glyph_index = FT_Get_Char_Index(ftFace, str[n]);

        int ret = FT_Load_Glyph(ftFace, glyph_index, FT_LOAD_DEFAULT | (antiAlias ? 0 : FT_LOAD_TARGET_MONO));

        if(ret != 0) { continue; }

        ret = FT_Render_Glyph(ftFace->glyph, antiAlias ? ft_render_mode_normal : ft_render_mode_mono);

        if(ret != 0) { continue; }

        FT_Bitmap bmp = slot->bitmap;

        uint08 *chr = bmp.buffer;

        int wmax = uiHeight*uiPitch - 1;
        int r = 0, w = (y0-slot->bitmap_top)*uiPitch + x0 + slot->bitmap_left;

        if(antiAlias)
        {
            for(int y=0;y<bmp.rows;y++)
            {
                for(int x=0;x<bmp.width;x++)
                {
                    if(w > 0 && (w < wmax))
                    {
                        pBuffer[w] = chr[r];
                    }

                    w++;
                    r++;
                }

                w += uiPitch - bmp.width;
                r += bmp.pitch - bmp.width;
            }
        }
        else
        {
            for(int y=0;y<bmp.rows;y++)
            {
                for(int x=0;x<bmp.width;x++)
                {
                    uint08 cur = getBit(chr, r++);

                    if(cur == 1 && (w > 0) && (w < wmax))
                    {
                        pBuffer[w] = c;
                    }

                    w++;
                }

                w += uiPitch - bmp.width;
                r += bmp.pitch*8 - bmp.width;
            }
        }

        x0 += slot->advance.x >> 6;
        y0 += slot->advance.y >> 6;
    }

    return;
}
#endif