// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#ifndef RENDERFX_H
#define RENDERFX_H

#include "BaseTypes.h"

/*! @brief Generic Sirenic effect
 * 
 * Represents a Sirenic effect. Frame by frame, an effect most provide
 * the ability to increment it's internal state machine via the Step()
 * method, and also must provide a Render() function that performs the
 * per-frame rendering. This is an Abstract Base Class.
 *
 */

class RenderFX
{
    public:

        /*! virtual deconstructor */
        virtual ~RenderFX() { }

        /*! Perform rendering based on current state */
        virtual bool Render() = 0;

        /*! Increment state forward one step */
        virtual bool Step() = 0;

    protected:

        /*! Protected constructor, to prevent direct instantiation of this base class */
        RenderFX(class Sirenic *pSirenic) : pSirenic(pSirenic) {}

        /*! Cached sirenic object */
        Sirenic *pSirenic;
};

/*! @brief Doubly linked list of render effects */
struct RenderFXNode
{
    /*! next render effect in list */
    RenderFXNode *pNext;

    /*! previous render effect in list */
    RenderFXNode *pPrev;

    /*! current render effect */
    RenderFX *pRenderFX;
};

#endif
