// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "TransformConfig.h"

/*! @todo: "negative bumpy vortex" - with negative pullStrength (looks less clean with current build) */
/*! @todo: "super roto-zoom" - stronger pull and/or variation in x/y */

//
// Simple
//

tc_Simple::tc_Simple()
{
}

void tc_Simple::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    return;
}

//
// Polar Waves
//

tc_PolarWaves::tc_PolarWaves()
{
    this->pullStrength   = RandomRange( 1.000f, 1.005f);
    this->rippleStrength = RandomRange( 0.005f, 0.015f);
    this->rippleCount    = RandomRange(-5.000f, 5.000f);
}

void tc_PolarWaves::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    dm_PolarPull(pullR, fR, pullStrength);
    dm_PolarWaves(pullR, pullD, fR, rippleCount, rippleStrength);
    return;
}

//
// Bumpy Vortex
//

tc_BumpyVortex::tc_BumpyVortex()
{
    this->pullStrength  = RandomRange( 1.000f,  1.005f);
    this->bumpStrength  = RandomRange( 1.000f,  2.000f);
    this->bumpCount     = RandomRange( 5.000f, 15.000f);
    this->twistStrength = RandomRange(-0.250f,  0.250f);

    if(RandomBool()) { this->bumpStrength = 0 - this->bumpStrength; }
}

void tc_BumpyVortex::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    dm_PolarPull(pullR, fR, pullStrength);
    dm_TwirlBump(pullR, pullD, fR, bumpCount, bumpStrength, twistStrength);
    return;
}

//
// Roto-Zoom
//

tc_RotoZoom::tc_RotoZoom()
{
    this->pullStrengthX = RandomRange(0.0010f, 0.0150f);
    this->pullStrengthY = this->pullStrengthX*RandomRange(0.75f, 1.25f);

    this->adjX = RandomRange(-0.005f, 0.005f);
    this->adjY = RandomRange(-0.005f, 0.005f);
}

void tc_RotoZoom::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    dm_RotoZoom(pullX, pullY, fX, fY, pullStrengthX, pullStrengthY);

    pullX += adjX;
    pullY += adjY;

    return;
}

//
// Corner Stretch
//

tc_CornerStretch::tc_CornerStretch()
{
	this->pullStrength = RandomRange(40, 400);
}

void tc_CornerStretch::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
	// 40 -> 400
	pullR += fabsf(fX*fY)/pullStrength;
    return;
}

//
// Quadrant Rotate
//

tc_QuadrantRotate::tc_QuadrantRotate()
{
	this->pullStrengthQuad1 = RandomRange(200, 1600);
	this->pullStrengthQuad2 = RandomRange(200, 1600);
	this->pullStrengthQuad3 = RandomRange(200, 1600);
	this->pullStrengthQuad4 = RandomRange(200, 1600);

    if(rand()%2 == 0) { this->pullStrengthQuad1 = 0 - this->pullStrengthQuad1; }
    if(rand()%2 == 0) { this->pullStrengthQuad2 = 0 - this->pullStrengthQuad2; }
    if(rand()%2 == 0) { this->pullStrengthQuad3 = 0 - this->pullStrengthQuad3; }
    if(rand()%2 == 0) { this->pullStrengthQuad4 = 0 - this->pullStrengthQuad4; }
}

void tc_QuadrantRotate::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    pullD += fabsf((fX+1.0f)*(fY-1.0f))/pullStrengthQuad1;
    pullD += fabsf((fX-1.0f)*(fY-1.0f))/pullStrengthQuad2;
    pullD += fabsf((fX-1.0f)*(fY+1.0f))/pullStrengthQuad3;
    pullD += fabsf((fX+1.0f)*(fY+1.0f))/pullStrengthQuad4;

    return;
}

//
// UnNamed
//

tc_UnNamed::tc_UnNamed()
{
}

void tc_UnNamed::Transform(float &pullX, float &pullY, float fX, float fY, float &pullR, float &pullD, float fR, float fD)
{
    //-1.0f := vanish
    // 1.0f := flat
    // inf  := etc!

    float fVanX = 0.0f, fVanY = 0.0f;
    float fPower = 0.005f;
    float fZ = 0.0f;

    /*! hazy pull out */
    //fZ += RandomRange(-fY*3+1, fY*3+1);

    //fZ += cosf(fX*3.1415f*4) - sinf(fY*3.1415f*4);
    fZ += sinf(fX*fY);

    /*! transform fZ */
    pullX += fX-(fX+fVanX)*(1.0f+fZ*fPower);
    pullY += fY-(fY+fVanY)*(1.0f+fZ*fPower);

    return;
}
