// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "SirenicWnd.h"

#include "resource.h"

SirenicWnd::SirenicWnd(uint width, uint height, Sirenic *pSirenic, bool bPopup) : RenderWnd(width, height, bPopup), pSirenic(pSirenic)
{
    this->szClassName = "SirenicWnd";
    this->szWindowName = "Sirenic " _SIRENIC_VERSION;

    this->hIcon = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SIRENIC));

    this->pRender = 0;
    this->hMenu = 0;

    return;
}

SirenicWnd::~SirenicWnd()
{
}

LRESULT CALLBACK SirenicWnd::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    static HWAVEIN hWaveIn = 0;
    static WAVEHDR waveHdr = { 0 };

    switch(uMsg)
    {
        case WM_CREATE:
        {
            hMenu = LoadMenu(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_SIRENIC_MENU));

            SetMenu(hWnd, hMenu);

            UpdateMenu();

            /* @todo waveIn !

            UINT numDev = waveInGetNumDevs();

            WAVEFORMATEX format;

            format.wFormatTag = WAVE_FORMAT_PCM;
            format.nChannels = 2;
            format.nSamplesPerSec = 44100;
            format.nAvgBytesPerSec = 2*44100*2;
            format.nBlockAlign = 2*2;
            format.wBitsPerSample = 16;
            format.cbSize = 0;

            MMRESULT mRet = waveInOpen(&hWaveIn, WAVE_MAPPER, &format, (DWORD)hWnd, NULL, CALLBACK_WINDOW | WAVE_FORMAT_DIRECT);

            memset(&waveHdr, 0, sizeof(waveHdr));

            waveHdr.lpData = (LPSTR)malloc(2*576*5);
            waveHdr.dwBufferLength = 2*576*5;

            mRet = waveInPrepareHeader(hWaveIn, &waveHdr, sizeof(waveHdr));
            mRet = waveInAddBuffer(hWaveIn, &waveHdr, sizeof(waveHdr));
            mRet = waveInStart(hWaveIn);

            pSirenic->EnableWaveform();
            pSirenic->UpdateWaveformColor(0.1f);
            pSirenic->UpdateWaveformScale(0.25f);
            */
        }
        break;

        /*
        case MM_WIM_DATA:
        {
            WAVEHDR *pWaveHdr = (WAVEHDR*)lParam;

            if(pWaveHdr->dwBytesRecorded < 2*576*5)
                break;

            uint08 *pData = (uint08*)pWaveHdr->lpData;

            pSirenic->UpdateWaveform(pData);

            waveInAddBuffer(hWaveIn, &waveHdr, sizeof(waveHdr));
        }
        break;
        */

        case WM_COMMAND:
        {
            switch(LOWORD(wParam))
            {
                case ID_FILE_EXIT:
                    SendMessage(hWnd, WM_CLOSE, 0, 0);
                    break;

                case ID_VIEW_FULLSCREEN:
                    SendMessage(hWnd, WM_LBUTTONDBLCLK, 0, 0);
                    break;

                case ID_VIEW_NEXTPALETTE_RANDOM_SMOOTH_HUE:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetNextPalette(CRENDER_PALETTE_RANDOM_SMOOTH_HUE);

                    UpdateMenu();
                }
                break;

                case ID_VIEW_NEXTPALETTE_BROKENTV:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetNextPalette(CRENDER_PALETTE_BROKEN_TV);

                    UpdateMenu();
                }
                break;

                case ID_VIEW_NEXTPALETTE_RANDOM_POLARIZED_HUE:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetNextPalette(CRENDER_PALETTE_RANDOM_POLARIZED_HUE);

                    UpdateMenu();
                }
                break;

                case ID_VIEW_NEXTPALETTE_RANDOM_POLARIZED_HUE2:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetNextPalette(CRENDER_PALETTE_RANDOM_POLARIZED_HUE2);

                    UpdateMenu();
                }
                break;

                case ID_VIEW_SPECIALEFFECTS_NONE:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetStereogramEnabled(false);

                    UpdateMenu();
                }
                break;

                case ID_VIEW_SPECIALEFFECTS_STEREOGRAM:
                {
                    if(pRender == 0)
                        break;

                    pRender->SetStereogramEnabled(true);

                    UpdateMenu();
                }
                break;

                case ID_RENDER_TRANSFORMMODE_BILINEAR:
                    pSirenic->SetExtraSrcWeightFlag(false);
                    UpdateMenu();
                    break;

                case ID_RENDER_TRANSFORMMODE_EXTRASOURCEWEIGHT:
                    pSirenic->SetExtraSrcWeightFlag(true);
                    UpdateMenu();
                    break;

                case ID_RENDER_DRAWMODE_SIMPLE:
                    pSirenic->SetAssignFunc(roAssignSimple);
                    UpdateMenu();
                    break;

                case ID_RENDER_DRAWMODE_AVERAGE:
                    pSirenic->SetAssignFunc(roAssignAverage);
                    UpdateMenu();
                    break;

                case ID_RENDER_DRAWMODE_ADDITIVE:
                    pSirenic->SetAssignFunc(roAssignAdd);
                    UpdateMenu();
                    break;

                case ID_HELP_ABOUT:
                {
                    MSGBOXPARAMS mbp;

                    memset(&mbp, 0, sizeof(mbp));

                    mbp.cbSize = sizeof(mbp);
                    mbp.hwndOwner = hWnd;
                    mbp.hInstance = GetModuleHandle(NULL);
                    mbp.lpszText = 
                        "    Sirenic Development Build (" _SIRENIC_VERSION ") by Caustik\n"
                        "    \n"
                        "    For more information, visit http://www.caustik.com/sirenic/   \n";
                    mbp.lpszCaption = "Sirenic " _SIRENIC_VERSION;
                    mbp.dwStyle = MB_OK | MB_USERICON;
                    mbp.lpszIcon = MAKEINTRESOURCE(IDI_SIRENIC);

                    MessageBoxIndirect(&mbp);
                }
                break;
            }
        }
        break;

        case WM_LBUTTONDBLCLK:
        {
            if(this->isMaximized)
            {
                LRESULT lRet = RenderWnd::WndProc(hWnd, uMsg, wParam, lParam);

                SetMenu(hWnd, hMenu);

                return lRet;
            }
            else
            {
                SetMenu(hWnd, NULL);
            }
        }
        break;
    }

    return RenderWnd::WndProc(hWnd, uMsg, wParam, lParam);
}

void SirenicWnd::UpdateMenu()
{
    bool eswf = pSirenic->GetExtraSrcWeightFlag();
    bool stereo = (pRender != 0 && pRender->GetStereogramEnabled());

    roAssignFunc assignFunc = pSirenic->GetAssignFunc();

    CRenderPalette nextPal = CRENDER_PALETTE_RANDOM_SMOOTH_HUE;

    if(pRender != 0) { nextPal = pRender->GetNextPalette(); }

    UpdateMenuItem(ID_VIEW_NEXTPALETTE_BROKENTV, nextPal == CRENDER_PALETTE_BROKEN_TV);
    UpdateMenuItem(ID_VIEW_NEXTPALETTE_RANDOM_SMOOTH_HUE, nextPal == CRENDER_PALETTE_RANDOM_SMOOTH_HUE);
    UpdateMenuItem(ID_VIEW_NEXTPALETTE_RANDOM_POLARIZED_HUE, nextPal == CRENDER_PALETTE_RANDOM_POLARIZED_HUE);
    UpdateMenuItem(ID_VIEW_NEXTPALETTE_RANDOM_POLARIZED_HUE2, nextPal == CRENDER_PALETTE_RANDOM_POLARIZED_HUE2);

    UpdateMenuItem(ID_VIEW_SPECIALEFFECTS_NONE, !stereo);
    UpdateMenuItem(ID_VIEW_SPECIALEFFECTS_STEREOGRAM, stereo);

    UpdateMenuItem(ID_RENDER_TRANSFORMMODE_BILINEAR, !eswf);
    UpdateMenuItem(ID_RENDER_TRANSFORMMODE_EXTRASOURCEWEIGHT, eswf);

    UpdateMenuItem(ID_RENDER_DRAWMODE_SIMPLE, (assignFunc == roAssignSimple));
    UpdateMenuItem(ID_RENDER_DRAWMODE_AVERAGE, (assignFunc == roAssignAverage));
    UpdateMenuItem(ID_RENDER_DRAWMODE_ADDITIVE, (assignFunc == roAssignAdd));

    return;
}

void SirenicWnd::UpdateMenuItem(int item, bool check)
{
    CheckMenuItem(hMenu, item, MF_BYCOMMAND | check ? MF_CHECKED : MF_UNCHECKED);
}
