// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "Sirenic.h"
#include "RenderWnd.h"
#include "CRenderDD4.h"
#include "CRenderDX9.h"
#include "CRenderDX9Texture.h"

#include "FX_PolarThreader.h"

#include <scrnsave.h>
#include <windows.h>
#include <time.h>

static DWORD WINAPI TransformThread(LPVOID lpThreadParameter);

/*! name of application */
extern "C" TCHAR szAppName[APPNAMEBUFFERLEN] = "Sirenic";

/*! width of fullscreen display */
static const int FULLSCREEN_WIDTH = 2560/4;
/*! height of fullscreen display */
static const int FULLSCREEN_HEIGHT = 1024/4;

/*! width of sirenic object */
static int sirenicWidth = FULLSCREEN_WIDTH;
/*! height of sirenic object */
static int sirenicHeight = FULLSCREEN_HEIGHT;

static const int SIRENIC_FPS = 40;
static const int SIRENIC_DELAY = (int)((1.0f / (float)SIRENIC_FPS)*1000.0f);

static Sirenic *pSirenic = NULL;
static CRender *pRender = NULL;

static bool killTransformThread = false;
static bool deadTransformThread = false;

static RECT WndRect = {0};

static void InitScreensaver(HWND hWnd);
static void DestroyScreensaver();

LRESULT WINAPI ScreenSaverProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_CREATE:
		{
			GetWindowRect(hWnd, &WndRect);

			if(fChildPreview)
			{
				sirenicWidth = 320;
				sirenicHeight = 240;
			}
			else
			{
				sirenicWidth = FULLSCREEN_WIDTH;
				sirenicHeight = FULLSCREEN_HEIGHT;
			}

			InitScreensaver(hWnd);
		}
		break;

		case WM_DESTROY:
			DestroyScreensaver();
			break;
	}

	return DefScreenSaverProc(hWnd, message, wParam, lParam);
}

static void InitScreensaver(HWND hWnd)
{
    timeBeginPeriod(1);

    pSirenic = new Sirenic(sirenicWidth, sirenicHeight, sirenicWidth);

    if(!pSirenic->Create())
    {
        MessageBox(NULL, "Failed to create sirenic object", "Sirenic", MB_OK);
        return;
    }

    srand(time(NULL));

    /*! Bezier */
    /*
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic, 255, CTF_SPIN_X, CTF_SPIN_Y));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic,   0, CTF_SPIN_Y, CTF_SPIN_X));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic, 255, CTF_SPIN_NEG_X, CTF_SPIN_NEG_Y));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic,   0, CTF_SPIN_NEG_Y, CTF_SPIN_NEG_X));
    //*/

    //
    // Polar threader!
    //
    //*
    for(float fSize = 1.0f; fSize > 0.25f; fSize -= 0.01f)
    {
        for(int r=0;r<3;r++)
        {
            float fColor = ((float)rand() / (float)RAND_MAX);

            uint08 color = (uint08)(fColor * 255.0f);

            RenderFXNode *pNode0 = pSirenic->InsertRenderFX(new FX_PolarThreader(pSirenic, fSize, color, 0.005f, 0.005f, false));
        }
    }
    //*/

    if(fChildPreview)
    {
        CRenderDD4 *pRenderDD4 = new CRenderDD4(sirenicWidth, sirenicHeight, hWnd);

        if(!pRenderDD4->Create())
        {
            MessageBox(NULL, "Failed to create renderer (DD4)", "Sirenic", MB_OK);
            return;
        }

        pRender = pRenderDD4;
    }
    else
    {
        CRenderDD4 *pRenderDD4 = new CRenderDD4(FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, hWnd);

        if(!pRenderDD4->Create())
        {
            MessageBox(NULL, "Failed to create renderer (DD4)", "Sirenic", MB_OK);
            return;
        }

        pRender = pRenderDD4;
    }

    // create transform thread
    {
        DWORD dwThreadId = 0;

        CreateThread(NULL, NULL, TransformThread, pSirenic, NULL, &dwThreadId);
    }
}

static void DestroyScreensaver()
{
    killTransformThread = true;

    while(!deadTransformThread)
    {
        Sleep(10);
    }

    delete pRender;

    delete pSirenic;

    timeEndPeriod(1);
}

DWORD WINAPI TransformThread(LPVOID lpThreadParameter)
{
    Sirenic *pSirenic = (Sirenic*)lpThreadParameter;

    int lastTime = timeGetTime();
    int skip = 0, lastFps = 0, lastDelta = 0;

    while(!killTransformThread)
    {
        int curTime = timeGetTime();
        int delta = curTime - lastTime;

        if(delta < SIRENIC_DELAY) 
        {
            Thread::Sleep(SIRENIC_DELAY - delta); 
            curTime = timeGetTime();
        }

        int fps = (int)((1.0 / ((float)curTime - (float)lastTime)) * 1000.0);

        lastTime = curTime;

        pSirenic->DrawRenderFX();
        pSirenic->DrawRenderFX();
        pSirenic->DrawRenderFX();
        pSirenic->DrawRenderFX();

        //
        // Render FPS
        //
		/*
        {
            static int last = 0;

            int cur = timeGetTime();

            char szBuffer[255];

            if(last != 0)
            {
                int fps = (int)((1.0 / ((float)cur - (float)last)) * 1000.0);

                static int lagFps = 0;
                static int chk = 0;

                if(chk++%10 == 0)
                    lagFps = fps;

                sprintf(szBuffer, "%s - ~%.02d FPS", "Sirenic " _SIRENIC_VERSION, lagFps);

                pSirenic->FillRect(0, 0, sirenicWidth, 21, 0);
                pSirenic->DrawString(5, 16, szBuffer, 16, true, 255);
            }

            last = cur;
        }
		//*/

        if(skip++%20 != 0) { fps = lastFps; delta = lastDelta; }

        lastFps = fps;
        lastDelta = delta;

        //*! render fps */
	    /*
        {
            char szBuffer[255];

            sprintf(szBuffer, "%s - ~%.02d FPS [Delay: %d ms]", "Sirenic " _SIRENIC_VERSION, fps, delta);

            pSirenic->FillRect(0, 0, sirenicWidth, 21, 0);
            pSirenic->DrawString(5, 16, szBuffer, 12, true, 255);
        }
	    //*/

        pSirenic->Transform();

        //*
        uint08 *pPixels = pSirenic->LockViewBuffer();

        if(!pRender->RenderPixels(pPixels))
            break;

        if(!pRender->RotatePalette())
            break;

        pSirenic->UnlockViewBuffer();

        pSirenic->AdvanceViewBuffer();
        //*/
    }

    deadTransformThread = true;

    return 0;
}

BOOL WINAPI ScreenSaverConfigureDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	MessageBox(NULL, "Configuration is not yet available", "Sirenic", MB_OK);
	return TRUE;
}

BOOL WINAPI RegisterDialogClasses(HANDLE hInst)
{
	return TRUE;
}
