// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "Sirenic.h"
#include "CRenderXbox.h"

#include <xtl.h>
#include <time.h>

static const int WIDTH  = 640;
static const int HEIGHT = 480;

static void CALLBACK TransformEvent(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2);

static bool killTransformEvent = false;
static UINT uiTransformEvent = 0;

static Sirenic     *pSirenic = NULL;
static CRenderXbox *pRender  = NULL;

/*! xbox entry point */
void __cdecl main()
{
    pSirenic = new Sirenic(WIDTH, HEIGHT, WIDTH);

    if(!pSirenic->Create())
    {
        //MessageBox(NULL, "Failed to create sirenic object", "Sirenic", MB_OK);
        return;
    }

    srand(time(NULL));

    //*
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic, 255, CTF_SPIN_X, CTF_SPIN_Y));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic,   0, CTF_SPIN_Y, CTF_SPIN_X));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic, 255, CTF_SPIN_NEG_X, CTF_SPIN_NEG_Y));
    pSirenic->InsertRenderFX(new FX_Bezier(pSirenic,   0, CTF_SPIN_NEG_Y, CTF_SPIN_NEG_X));
    //*/

    /*
    for(float fSize = 1.0f; fSize > 0.25f; fSize -= 0.01f)
    {
        for(int r=0;r<3;r++)
        {
            float fColor = ((float)rand() / (float)RAND_MAX);

            uint08 color = (uint08)(fColor * 255.0f);

            RenderFXNode *pNode0 = pSirenic->InsertRenderFX(new FX_PolarThreader(pSirenic, fSize, color, 0.035f, 0.025f, true));
        }
    }
    //*/

    RenderFXNode *pNode0 = pSirenic->InsertRenderFX(new FX_PolyDance(pSirenic, 0.01f, 0.75f, 0, 0.75f, 0, 1.0f, 0.001f));
    
    //pSirenic->InsertRenderFX(new FX_TreeFractal(pSirenic));

    pRender = new CRenderXbox(WIDTH, HEIGHT);

    if(!pRender->Create())
    {
        //MessageBox(NULL, "Failed to create renderer (Xbox)", "Sirenic", MB_OK);
        return;
    }

    /*! begin transform events */
    {
        static const int uiFPS = 60;
        static const int uiDelay = (int)((1.0f / (float)uiFPS)*1000.0f);

        uiTransformEvent = timeSetEvent(uiDelay, 1, TransformEvent, 0, TIME_PERIODIC);
    }

    /*! never terminate */
    while(true) { Sleep(20); }

    killTransformEvent = true;

    timeKillEvent(uiTransformEvent);

    delete pRender;

    delete pSirenic;

    return;
}

static void CALLBACK TransformEvent(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
    if(killTransformEvent)
        return;

    static UINT uiQueueCount = 0;

    static HANDLE hMutex = CreateMutex(NULL, FALSE, NULL);

    // prevent too many events from firing at once
    if(uiQueueCount > 3) { return; }

    uiQueueCount++;

    WaitForSingleObject(hMutex, INFINITE);

    pSirenic->DrawRenderFX();
    pSirenic->DrawRenderFX();

    /*! respond to mouse movement */
    {
        float mouseX = 0.0f;;//(float)pWindow->GetMousePosX();
        float mouseY = 0.0f;//(float)pWindow->GetMousePosY();

        mouseX /= 640.0f;//pWindow->GetWidth();
        mouseY /= 480.0f;//pWindow->GetHeight();

        mouseX *= WIDTH;
        mouseY *= HEIGHT;

        pSirenic->SetMousePos((uint)mouseX, (uint)mouseY);
        pSirenic->SetMouseDownL(false);//pWindow->GetMouseDownL());

        if(pSirenic->GetMouseDownL())
        {
            //pSirenic->DrawRectWu(mouseX-8, mouseY-8, mouseX+8, mouseY+8, 1.0);
            //pSirenic->DrawRect((int)mouseX-8, (int)mouseY-8, (int)mouseX+8, (int)mouseY+8, 255);

            //pSirenic->DrawRect((int)mouseX-4, (int)mouseY-4, (int)mouseX+4, (int)mouseY+4, 255);
            //pSirenic->DrawRect((int)mouseX-2, (int)mouseY-2, (int)mouseX+2, (int)mouseY+2, 255);

            //pSirenic->DrawRect((int)mouseX-1, (int)mouseY-1, (int)mouseX+1, (int)mouseY+1, 255);

            //pSirenic->DrawString(mouseX, mouseY, "Sirenic", 64, false, 255);
        }
    }

    pSirenic->Transform();

    //*
    uint08 *pPixels = pSirenic->LockViewBuffer();

    if(!pRender->RenderPixels(pPixels))
    {
        ReleaseMutex(hMutex);
        return;
    }

    if(!pRender->RotatePalette())
    {
        ReleaseMutex(hMutex);
        return;
    }

    pSirenic->UnlockViewBuffer();

    pSirenic->AdvanceViewBuffer();
    //*/

    uiQueueCount--;

    ReleaseMutex(hMutex);
    
    return;
}
