// ******************************************************************
// *
// *  Sirenic is free software; you can redistribute it and/or modify
// *  it under the terms of the GNU General Public License as published
// *  by the Free Software Foundation; either version 2 of the license,
// *  or (at your option) any later version.
// *
// *  This program is distributed in the hope that it will be useful,
// *  but WITHOUT ANY WARRANTY; without even the implied warranty of
// *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// *  GNU General Public License for more details.
// *
// *  You should have recieved a copy of the GNU General Public License
// *  along with this program; see the file COPYING.
// *  If not, write to the Free Software Foundation, Inc.,
// *  59 Temple Place - Suite 330, Bostom, MA 02111-1307, USA.
// *
// *  (c) 2002-2004 Aaron Robinson <caustik@caustik.com>
// *
// *  All rights reserved
// *
// ******************************************************************

#include "vis_sirenic.h"

static winampVisModule *getModule(int which);

static void config(winampVisModule *this_mod);
static int init(winampVisModule *this_mod);
static int render(winampVisModule *this_mod);
static void quit(winampVisModule *this_mod);

// main header
static winampVisHeader visHeader = { VIS_HDRVER, "Sirenic " _SIRENIC_VERSION, getModule };

// primary visualization module
static winampVisModule visModule =
{
    "Sirenic " _SIRENIC_VERSION,
    NULL,	// hwndParent
    NULL,	// hDllInstance
    0,		// sRate
    0,		// nCh
    15,		// latencyMS
    15,		// delayMS
    0,		// spectrumNch
    2,		// waveformNch
    { 0, },	// spectrumData
    { 0, },	// waveformData
    config, init, render, quit
};

// this is the only exported symbol. returns our main header.
extern "C" __declspec( dllexport ) winampVisHeader *winampVisGetHeader() 
{ 
    return &visHeader; 
}

// we only expose one visualzation module
static winampVisModule *getModule(int which)
{
    if(which == 0) { return &visModule; }

    return NULL;
}

static void config(winampVisModule *this_mod)
{
    MessageBox(this_mod->hwndParent, "Currently there is no configuration available", "Sirenic " _SIRENIC_VERSION, MB_OK);

    return;
}

static int init(winampVisModule *this_mod)
{
    vis_sirenic *pVisSirenic = new vis_sirenic(this_mod);

    if(!pVisSirenic->Create())
    {
        return 1;
    }

    this_mod->userData = pVisSirenic;

    return 0;
}

static int render(winampVisModule *this_mod)
{
    vis_sirenic *pVisSirenic = (vis_sirenic*)this_mod->userData;

    pVisSirenic->Render();

    return 0;
}

static void quit(winampVisModule *this_mod)
{
    vis_sirenic *pVisSirenic = (vis_sirenic*)this_mod->userData;

    delete pVisSirenic;

    SendMessage(visModule.hwndParent, WM_WA_IPC, NULL, IPC_SETVISWND);

    return;
}
